"""
This type stub file was generated by pyright.
"""

import datetime
import numpy as np
from collections import abc
from typing import Any, Final, Hashable, Literal, Sequence, TYPE_CHECKING
from pandas._typing import CompressionOptions, FilePath, ReadBuffer, StorageOptions, WriteBuffer
from pandas.util._decorators import Appender, deprecate_nonkeyword_arguments, doc
from pandas.core.frame import DataFrame
from pandas.core.series import Series
from pandas.core.shared_docs import _shared_docs

"""
Module contains tools for processing Stata files into DataFrames

The StataReader below was originally written by Joe Presbrey as part of PyDTA.
It has been extended and improved by Skipper Seabold from the Statsmodels
project who also developed the StataWriter and was finally added to pandas in
a once again improved version.

You can find more information on http://presbrey.mit.edu/PyDTA and
https://www.statsmodels.org/devel/
"""
if TYPE_CHECKING:
    ...
_version_error = ...
_statafile_processing_params1 = ...
_statafile_processing_params2 = ...
_chunksize_params = ...
_iterator_params = ...
_reader_notes = ...
_read_stata_doc = ...
_read_method_doc = ...
_stata_reader_doc = ...
_date_formats = ...
stata_epoch: Final = ...
excessive_string_length_error: Final = ...
precision_loss_doc: Final = ...
value_label_mismatch_doc: Final = ...
invalid_name_doc: Final = ...
categorical_conversion_warning: Final = ...
class StataValueLabel:
    """
    Parse a categorical column and prepare formatted output

    Parameters
    ----------
    catarray : Series
        Categorical Series to encode
    encoding : {"latin-1", "utf-8"}
        Encoding to use for value labels.
    """
    def __init__(self, catarray: Series, encoding: Literal["latin-1", "utf-8"] = ...) -> None:
        ...
    
    def generate_value_label(self, byteorder: str) -> bytes:
        """
        Generate the binary representation of the value labels.

        Parameters
        ----------
        byteorder : str
            Byte order of the output

        Returns
        -------
        value_label : bytes
            Bytes containing the formatted value label
        """
        ...
    


class StataNonCatValueLabel(StataValueLabel):
    """
    Prepare formatted version of value labels

    Parameters
    ----------
    labname : str
        Value label name
    value_labels: Dictionary
        Mapping of values to labels
    encoding : {"latin-1", "utf-8"}
        Encoding to use for value labels.
    """
    def __init__(self, labname: str, value_labels: dict[float, str], encoding: Literal["latin-1", "utf-8"] = ...) -> None:
        ...
    


class StataMissingValue:
    """
    An observation's missing value.

    Parameters
    ----------
    value : {int, float}
        The Stata missing value code

    Notes
    -----
    More information: <https://www.stata.com/help.cgi?missing>

    Integer missing values make the code '.', '.a', ..., '.z' to the ranges
    101 ... 127 (for int8), 32741 ... 32767  (for int16) and 2147483621 ...
    2147483647 (for int32).  Missing values for floating point data types are
    more complex but the pattern is simple to discern from the following table.

    np.float32 missing values (float in Stata)
    0000007f    .
    0008007f    .a
    0010007f    .b
    ...
    00c0007f    .x
    00c8007f    .y
    00d0007f    .z

    np.float64 missing values (double in Stata)
    000000000000e07f    .
    000000000001e07f    .a
    000000000002e07f    .b
    ...
    000000000018e07f    .x
    000000000019e07f    .y
    00000000001ae07f    .z
    """
    MISSING_VALUES: dict[float, str] = ...
    bases: Final = ...
    float32_base: bytes = ...
    increment: int = ...
    float64_base: bytes = ...
    increment = ...
    BASE_MISSING_VALUES: Final = ...
    def __init__(self, value: float) -> None:
        ...
    
    @property
    def string(self) -> str:
        """
        The Stata representation of the missing value: '.', '.a'..'.z'

        Returns
        -------
        str
            The representation of the missing value.
        """
        ...
    
    @property
    def value(self) -> float:
        """
        The binary representation of the missing value.

        Returns
        -------
        {int, float}
            The binary representation of the missing value.
        """
        ...
    
    def __str__(self) -> str:
        ...
    
    def __repr__(self) -> str:
        ...
    
    def __eq__(self, other: Any) -> bool:
        ...
    
    @classmethod
    def get_base_missing_value(cls, dtype: np.dtype) -> float:
        ...
    


class StataParser:
    def __init__(self) -> None:
        ...
    


class StataReader(StataParser, abc.Iterator):
    __doc__ = ...
    def __init__(self, path_or_buf: FilePath | ReadBuffer[bytes], convert_dates: bool = ..., convert_categoricals: bool = ..., index_col: str | None = ..., convert_missing: bool = ..., preserve_dtypes: bool = ..., columns: Sequence[str] | None = ..., order_categoricals: bool = ..., chunksize: int | None = ..., compression: CompressionOptions = ..., storage_options: StorageOptions = ...) -> None:
        ...
    
    def __enter__(self) -> StataReader:
        """enter context manager"""
        ...
    
    def __exit__(self, exc_type, exc_value, traceback) -> None:
        """exit context manager"""
        ...
    
    def close(self) -> None:
        """close the handle if its open"""
        ...
    
    def __next__(self) -> DataFrame:
        ...
    
    def get_chunk(self, size: int | None = ...) -> DataFrame:
        """
        Reads lines from Stata file and returns as dataframe

        Parameters
        ----------
        size : int, defaults to None
            Number of lines to read.  If None, reads whole file.

        Returns
        -------
        DataFrame
        """
        ...
    
    @Appender(_read_method_doc)
    def read(self, nrows: int | None = ..., convert_dates: bool | None = ..., convert_categoricals: bool | None = ..., index_col: str | None = ..., convert_missing: bool | None = ..., preserve_dtypes: bool | None = ..., columns: Sequence[str] | None = ..., order_categoricals: bool | None = ...) -> DataFrame:
        ...
    
    @property
    def data_label(self) -> str:
        """
        Return data label of Stata file.
        """
        ...
    
    def variable_labels(self) -> dict[str, str]:
        """
        Return a dict associating each variable name with corresponding label.

        Returns
        -------
        dict
        """
        ...
    
    def value_labels(self) -> dict[str, dict[float, str]]:
        """
        Return a nested dict associating each variable name to its value and label.

        Returns
        -------
        dict
        """
        ...
    


@Appender(_read_stata_doc)
@deprecate_nonkeyword_arguments(version=None, allowed_args=["filepath_or_buffer"])
def read_stata(filepath_or_buffer: FilePath | ReadBuffer[bytes], convert_dates: bool = ..., convert_categoricals: bool = ..., index_col: str | None = ..., convert_missing: bool = ..., preserve_dtypes: bool = ..., columns: Sequence[str] | None = ..., order_categoricals: bool = ..., chunksize: int | None = ..., iterator: bool = ..., compression: CompressionOptions = ..., storage_options: StorageOptions = ...) -> DataFrame | StataReader:
    ...

@doc(storage_options=_shared_docs["storage_options"], compression_options=_shared_docs["compression_options"] % "fname")
class StataWriter(StataParser):
    """
    A class for writing Stata binary dta files

    Parameters
    ----------
    fname : path (string), buffer or path object
        string, path object (pathlib.Path or py._path.local.LocalPath) or
        object implementing a binary write() functions. If using a buffer
        then the buffer will not be automatically closed after the file
        is written.
    data : DataFrame
        Input to save
    convert_dates : dict
        Dictionary mapping columns containing datetime types to stata internal
        format to use when writing the dates. Options are 'tc', 'td', 'tm',
        'tw', 'th', 'tq', 'ty'. Column can be either an integer or a name.
        Datetime columns that do not have a conversion type specified will be
        converted to 'tc'. Raises NotImplementedError if a datetime column has
        timezone information
    write_index : bool
        Write the index to Stata dataset.
    byteorder : str
        Can be ">", "<", "little", or "big". default is `sys.byteorder`
    time_stamp : datetime
        A datetime to use as file creation date.  Default is the current time
    data_label : str
        A label for the data set.  Must be 80 characters or smaller.
    variable_labels : dict
        Dictionary containing columns as keys and variable labels as values.
        Each label must be 80 characters or smaller.
    {compression_options}

        .. versionadded:: 1.1.0

        .. versionchanged:: 1.4.0 Zstandard support.

    {storage_options}

        .. versionadded:: 1.2.0

    value_labels : dict of dicts
        Dictionary containing columns as keys and dictionaries of column value
        to labels as values. The combined length of all labels for a single
        variable must be 32,000 characters or smaller.

        .. versionadded:: 1.4.0

    Returns
    -------
    writer : StataWriter instance
        The StataWriter instance has a write_file method, which will
        write the file to the given `fname`.

    Raises
    ------
    NotImplementedError
        * If datetimes contain timezone information
    ValueError
        * Columns listed in convert_dates are neither datetime64[ns]
          or datetime.datetime
        * Column dtype is not representable in Stata
        * Column listed in convert_dates is not in DataFrame
        * Categorical label contains more than 32,000 characters

    Examples
    --------
    >>> data = pd.DataFrame([[1.0, 1]], columns=['a', 'b'])
    >>> writer = StataWriter('./data_file.dta', data)
    >>> writer.write_file()

    Directly write a zip file
    >>> compression = {{"method": "zip", "archive_name": "data_file.dta"}}
    >>> writer = StataWriter('./data_file.zip', data, compression=compression)
    >>> writer.write_file()

    Save a DataFrame with dates
    >>> from datetime import datetime
    >>> data = pd.DataFrame([[datetime(2000,1,1)]], columns=['date'])
    >>> writer = StataWriter('./date_data_file.dta', data, {{'date' : 'tw'}})
    >>> writer.write_file()
    """
    _max_string_length = ...
    _encoding: Literal["latin-1", "utf-8"] = ...
    def __init__(self, fname: FilePath | WriteBuffer[bytes], data: DataFrame, convert_dates: dict[Hashable, str] | None = ..., write_index: bool = ..., byteorder: str | None = ..., time_stamp: datetime.datetime | None = ..., data_label: str | None = ..., variable_labels: dict[Hashable, str] | None = ..., compression: CompressionOptions = ..., storage_options: StorageOptions = ..., *, value_labels: dict[Hashable, dict[float, str]] | None = ...) -> None:
        ...
    
    def write_file(self) -> None:
        """
        Export DataFrame object to Stata dta format.
        """
        ...
    


class StataStrLWriter:
    """
    Converter for Stata StrLs

    Stata StrLs map 8 byte values to strings which are stored using a
    dictionary-like format where strings are keyed to two values.

    Parameters
    ----------
    df : DataFrame
        DataFrame to convert
    columns : Sequence[str]
        List of columns names to convert to StrL
    version : int, optional
        dta version.  Currently supports 117, 118 and 119
    byteorder : str, optional
        Can be ">", "<", "little", or "big". default is `sys.byteorder`

    Notes
    -----
    Supports creation of the StrL block of a dta file for dta versions
    117, 118 and 119.  These differ in how the GSO is stored.  118 and
    119 store the GSO lookup value as a uint32 and a uint64, while 117
    uses two uint32s. 118 and 119 also encode all strings as unicode
    which is required by the format.  117 uses 'latin-1' a fixed width
    encoding that extends the 7-bit ascii table with an additional 128
    characters.
    """
    def __init__(self, df: DataFrame, columns: Sequence[str], version: int = ..., byteorder: str | None = ...) -> None:
        ...
    
    def generate_table(self) -> tuple[dict[str, tuple[int, int]], DataFrame]:
        """
        Generates the GSO lookup table for the DataFrame

        Returns
        -------
        gso_table : dict
            Ordered dictionary using the string found as keys
            and their lookup position (v,o) as values
        gso_df : DataFrame
            DataFrame where strl columns have been converted to
            (v,o) values

        Notes
        -----
        Modifies the DataFrame in-place.

        The DataFrame returned encodes the (v,o) values as uint64s. The
        encoding depends on the dta version, and can be expressed as

        enc = v + o * 2 ** (o_size * 8)

        so that v is stored in the lower bits and o is in the upper
        bits. o_size is

          * 117: 4
          * 118: 6
          * 119: 5
        """
        ...
    
    def generate_blob(self, gso_table: dict[str, tuple[int, int]]) -> bytes:
        """
        Generates the binary blob of GSOs that is written to the dta file.

        Parameters
        ----------
        gso_table : dict
            Ordered dictionary (str, vo)

        Returns
        -------
        gso : bytes
            Binary content of dta file to be placed between strl tags

        Notes
        -----
        Output format depends on dta version.  117 uses two uint32s to
        express v and o while 118+ uses a uint32 for v and a uint64 for o.
        """
        ...
    


class StataWriter117(StataWriter):
    """
    A class for writing Stata binary dta files in Stata 13 format (117)

    Parameters
    ----------
    fname : path (string), buffer or path object
        string, path object (pathlib.Path or py._path.local.LocalPath) or
        object implementing a binary write() functions. If using a buffer
        then the buffer will not be automatically closed after the file
        is written.
    data : DataFrame
        Input to save
    convert_dates : dict
        Dictionary mapping columns containing datetime types to stata internal
        format to use when writing the dates. Options are 'tc', 'td', 'tm',
        'tw', 'th', 'tq', 'ty'. Column can be either an integer or a name.
        Datetime columns that do not have a conversion type specified will be
        converted to 'tc'. Raises NotImplementedError if a datetime column has
        timezone information
    write_index : bool
        Write the index to Stata dataset.
    byteorder : str
        Can be ">", "<", "little", or "big". default is `sys.byteorder`
    time_stamp : datetime
        A datetime to use as file creation date.  Default is the current time
    data_label : str
        A label for the data set.  Must be 80 characters or smaller.
    variable_labels : dict
        Dictionary containing columns as keys and variable labels as values.
        Each label must be 80 characters or smaller.
    convert_strl : list
        List of columns names to convert to Stata StrL format.  Columns with
        more than 2045 characters are automatically written as StrL.
        Smaller columns can be converted by including the column name.  Using
        StrLs can reduce output file size when strings are longer than 8
        characters, and either frequently repeated or sparse.
    {compression_options}

        .. versionadded:: 1.1.0

        .. versionchanged:: 1.4.0 Zstandard support.

    value_labels : dict of dicts
        Dictionary containing columns as keys and dictionaries of column value
        to labels as values. The combined length of all labels for a single
        variable must be 32,000 characters or smaller.

        .. versionadded:: 1.4.0

    Returns
    -------
    writer : StataWriter117 instance
        The StataWriter117 instance has a write_file method, which will
        write the file to the given `fname`.

    Raises
    ------
    NotImplementedError
        * If datetimes contain timezone information
    ValueError
        * Columns listed in convert_dates are neither datetime64[ns]
          or datetime.datetime
        * Column dtype is not representable in Stata
        * Column listed in convert_dates is not in DataFrame
        * Categorical label contains more than 32,000 characters

    Examples
    --------
    >>> data = pd.DataFrame([[1.0, 1, 'a']], columns=['a', 'b', 'c'])
    >>> writer = pd.io.stata.StataWriter117('./data_file.dta', data)
    >>> writer.write_file()

    Directly write a zip file
    >>> compression = {"method": "zip", "archive_name": "data_file.dta"}
    >>> writer = pd.io.stata.StataWriter117(
    ...     './data_file.zip', data, compression=compression
    ...     )
    >>> writer.write_file()

    Or with long strings stored in strl format
    >>> data = pd.DataFrame([['A relatively long string'], [''], ['']],
    ...                     columns=['strls'])
    >>> writer = pd.io.stata.StataWriter117(
    ...     './data_file_with_long_strings.dta', data, convert_strl=['strls'])
    >>> writer.write_file()
    """
    _max_string_length = ...
    _dta_version = ...
    def __init__(self, fname: FilePath | WriteBuffer[bytes], data: DataFrame, convert_dates: dict[Hashable, str] | None = ..., write_index: bool = ..., byteorder: str | None = ..., time_stamp: datetime.datetime | None = ..., data_label: str | None = ..., variable_labels: dict[Hashable, str] | None = ..., convert_strl: Sequence[Hashable] | None = ..., compression: CompressionOptions = ..., storage_options: StorageOptions = ..., *, value_labels: dict[Hashable, dict[float, str]] | None = ...) -> None:
        ...
    


class StataWriterUTF8(StataWriter117):
    """
    Stata binary dta file writing in Stata 15 (118) and 16 (119) formats

    DTA 118 and 119 format files support unicode string data (both fixed
    and strL) format. Unicode is also supported in value labels, variable
    labels and the dataset label. Format 119 is automatically used if the
    file contains more than 32,767 variables.

    .. versionadded:: 1.0.0

    Parameters
    ----------
    fname : path (string), buffer or path object
        string, path object (pathlib.Path or py._path.local.LocalPath) or
        object implementing a binary write() functions. If using a buffer
        then the buffer will not be automatically closed after the file
        is written.
    data : DataFrame
        Input to save
    convert_dates : dict, default None
        Dictionary mapping columns containing datetime types to stata internal
        format to use when writing the dates. Options are 'tc', 'td', 'tm',
        'tw', 'th', 'tq', 'ty'. Column can be either an integer or a name.
        Datetime columns that do not have a conversion type specified will be
        converted to 'tc'. Raises NotImplementedError if a datetime column has
        timezone information
    write_index : bool, default True
        Write the index to Stata dataset.
    byteorder : str, default None
        Can be ">", "<", "little", or "big". default is `sys.byteorder`
    time_stamp : datetime, default None
        A datetime to use as file creation date.  Default is the current time
    data_label : str, default None
        A label for the data set.  Must be 80 characters or smaller.
    variable_labels : dict, default None
        Dictionary containing columns as keys and variable labels as values.
        Each label must be 80 characters or smaller.
    convert_strl : list, default None
        List of columns names to convert to Stata StrL format.  Columns with
        more than 2045 characters are automatically written as StrL.
        Smaller columns can be converted by including the column name.  Using
        StrLs can reduce output file size when strings are longer than 8
        characters, and either frequently repeated or sparse.
    version : int, default None
        The dta version to use. By default, uses the size of data to determine
        the version. 118 is used if data.shape[1] <= 32767, and 119 is used
        for storing larger DataFrames.
    {compression_options}

        .. versionadded:: 1.1.0

        .. versionchanged:: 1.4.0 Zstandard support.

    value_labels : dict of dicts
        Dictionary containing columns as keys and dictionaries of column value
        to labels as values. The combined length of all labels for a single
        variable must be 32,000 characters or smaller.

        .. versionadded:: 1.4.0

    Returns
    -------
    StataWriterUTF8
        The instance has a write_file method, which will write the file to the
        given `fname`.

    Raises
    ------
    NotImplementedError
        * If datetimes contain timezone information
    ValueError
        * Columns listed in convert_dates are neither datetime64[ns]
          or datetime.datetime
        * Column dtype is not representable in Stata
        * Column listed in convert_dates is not in DataFrame
        * Categorical label contains more than 32,000 characters

    Examples
    --------
    Using Unicode data and column names

    >>> from pandas.io.stata import StataWriterUTF8
    >>> data = pd.DataFrame([[1.0, 1, 'ᴬ']], columns=['a', 'β', 'ĉ'])
    >>> writer = StataWriterUTF8('./data_file.dta', data)
    >>> writer.write_file()

    Directly write a zip file
    >>> compression = {"method": "zip", "archive_name": "data_file.dta"}
    >>> writer = StataWriterUTF8('./data_file.zip', data, compression=compression)
    >>> writer.write_file()

    Or with long strings stored in strl format

    >>> data = pd.DataFrame([['ᴀ relatively long ŝtring'], [''], ['']],
    ...                     columns=['strls'])
    >>> writer = StataWriterUTF8('./data_file_with_long_strings.dta', data,
    ...                          convert_strl=['strls'])
    >>> writer.write_file()
    """
    _encoding: Literal["utf-8"] = ...
    def __init__(self, fname: FilePath | WriteBuffer[bytes], data: DataFrame, convert_dates: dict[Hashable, str] | None = ..., write_index: bool = ..., byteorder: str | None = ..., time_stamp: datetime.datetime | None = ..., data_label: str | None = ..., variable_labels: dict[Hashable, str] | None = ..., convert_strl: Sequence[Hashable] | None = ..., version: int | None = ..., compression: CompressionOptions = ..., storage_options: StorageOptions = ..., *, value_labels: dict[Hashable, dict[float, str]] | None = ...) -> None:
        ...
    





"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas.core.arrays.categorical import Categorical
from pandas.core.indexes.api import CategoricalIndex

if TYPE_CHECKING:
    ...
def recode_for_groupby(c: Categorical, sort: bool, observed: bool) -> tuple[Categorical, Categorical | None]:
    """
    Code the categories to ensure we can groupby for categoricals.

    If observed=True, we return a new Categorical with the observed
    categories only.

    If sort=False, return a copy of self, coded with categories as
    returned by .unique(), followed by any categories not appearing in
    the data. If sort=True, return self.

    This method is needed solely to ensure the categorical index of the
    GroupBy result has categories in the order of appearance in the data
    (GH-8868).

    Parameters
    ----------
    c : Categorical
    sort : bool
        The value of the sort parameter groupby was called with.
    observed : bool
        Account only for the observed values

    Returns
    -------
    Categorical
        If sort=False, the new categories are set to the order of
        appearance in codes (unless ordered=True, in which case the
        original order is preserved), followed by any unrepresented
        categories in the original order.
    Categorical or None
        If we are observed, return the original categorical, otherwise None
    """
    ...

def recode_from_groupby(c: Categorical, sort: bool, ci: CategoricalIndex) -> CategoricalIndex:
    """
    Reverse the codes_to_groupby to account for sort / observed.

    Parameters
    ----------
    c : Categorical
    sort : bool
        The value of the sort parameter groupby was called with.
    ci : CategoricalIndex
        The codes / categories to recode

    Returns
    -------
    CategoricalIndex
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Iterator, Sequence, final, overload
from pandas._typing import ArrayLike, T, npt
from pandas import Index
from pandas.core.arrays._mixins import NDArrayBackedExtensionArray
from pandas.core.internals.blocks import Block as B

def slice_len(slc: slice, objlen: int = ...) -> int:
    ...

def get_blkno_indexers(blknos: np.ndarray, group: bool = ...) -> list[tuple[int, slice | np.ndarray]]:
    ...

def get_blkno_placements(blknos: np.ndarray, group: bool = ...) -> Iterator[tuple[int, BlockPlacement]]:
    ...

def update_blklocs_and_blknos(blklocs: npt.NDArray[np.intp], blknos: npt.NDArray[np.intp], loc: int, nblocks: int) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

@final
class BlockPlacement:
    def __init__(self, val: int | slice | np.ndarray) -> None:
        ...
    
    @property
    def indexer(self) -> np.ndarray | slice:
        ...
    
    @property
    def as_array(self) -> np.ndarray:
        ...
    
    @property
    def as_slice(self) -> slice:
        ...
    
    @property
    def is_slice_like(self) -> bool:
        ...
    
    @overload
    def __getitem__(self, loc: slice | Sequence[int]) -> BlockPlacement:
        ...
    
    @overload
    def __getitem__(self, loc: int) -> int:
        ...
    
    def __iter__(self) -> Iterator[int]:
        ...
    
    def __len__(self) -> int:
        ...
    
    def delete(self, loc) -> BlockPlacement:
        ...
    
    def append(self, others: list[BlockPlacement]) -> BlockPlacement:
        ...
    
    def tile_for_unstack(self, factor: int) -> npt.NDArray[np.intp]:
        ...
    


class SharedBlock:
    _mgr_locs: BlockPlacement
    ndim: int
    values: ArrayLike
    def __init__(self, values: ArrayLike, placement: BlockPlacement, ndim: int) -> None:
        ...
    


class NumpyBlock(SharedBlock):
    values: np.ndarray
    @final
    def getitem_block_index(self: T, slicer: slice) -> T:
        ...
    


class NDArrayBackedBlock(SharedBlock):
    values: NDArrayBackedExtensionArray
    @final
    def getitem_block_index(self: T, slicer: slice) -> T:
        ...
    


class Block(SharedBlock):
    ...


class BlockManager:
    blocks: tuple[B, ...]
    axes: list[Index]
    _known_consolidated: bool
    _is_consolidated: bool
    _blknos: np.ndarray
    _blklocs: np.ndarray
    def __init__(self, blocks: tuple[B, ...], axes: list[Index], verify_integrity=...) -> None:
        ...
    
    def get_slice(self: T, slobj: slice, axis: int = ...) -> T:
        ...
    





"""
This type stub file was generated by pyright.
"""

from contextlib import contextmanager
from typing import Callable, Iterator
from pandas._typing import F

"""
This module provides decorator functions which can be applied to test objects
in order to skip those objects when certain conditions occur. A sample use case
is to detect if the platform is missing ``matplotlib``. If so, any test objects
which require ``matplotlib`` and decorated with ``@td.skip_if_no_mpl`` will be
skipped by ``pytest`` during the execution of the test suite.

To illustrate, after importing this module:

import pandas.util._test_decorators as td

The decorators can be applied to classes:

@td.skip_if_some_reason
class Foo:
    ...

Or individual functions:

@td.skip_if_some_reason
def test_foo():
    ...

For more information, refer to the ``pytest`` documentation on ``skipif``.
"""
def safe_import(mod_name: str, min_version: str | None = ...): # -> Any | Literal[False]:
    """
    Parameters
    ----------
    mod_name : str
        Name of the module to be imported
    min_version : str, default None
        Minimum required version of the specified mod_name

    Returns
    -------
    object
        The imported module if successful, or False
    """
    ...

def skip_if_installed(package: str):
    """
    Skip a test if a package is installed.

    Parameters
    ----------
    package : str
        The name of the package.
    """
    ...

def skip_if_no(package: str, min_version: str | None = ...):
    """
    Generic function to help skip tests when required packages are not
    present on the testing system.

    This function returns a pytest mark with a skip condition that will be
    evaluated during test collection. An attempt will be made to import the
    specified ``package`` and optionally ensure it meets the ``min_version``

    The mark can be used as either a decorator for a test function or to be
    applied to parameters in pytest.mark.parametrize calls or parametrized
    fixtures.

    If the import and version check are unsuccessful, then the test function
    (or test case when used in conjunction with parametrization) will be
    skipped.

    Parameters
    ----------
    package: str
        The name of the required package.
    min_version: str or None, default None
        Optional minimum version of the package.

    Returns
    -------
    _pytest.mark.structures.MarkDecorator
        a pytest.mark.skipif to use as either a test decorator or a
        parametrization mark.
    """
    ...

skip_if_no_mpl = ...
skip_if_mpl = ...
skip_if_32bit = ...
skip_if_windows = ...
skip_if_not_us_locale = ...
skip_if_no_scipy = ...
skip_if_no_ne = ...
def skip_if_np_lt(ver_str: str, *args, reason: str | None = ...):
    ...

def parametrize_fixture_doc(*args) -> Callable[[F], F]:
    """
    Intended for use as a decorator for parametrized fixture,
    this function will wrap the decorated function with a pytest
    ``parametrize_fixture_doc`` mark. That mark will format
    initial fixture docstring by replacing placeholders {0}, {1} etc
    with parameters passed as arguments.

    Parameters
    ----------
    args: iterable
        Positional arguments for docstring.

    Returns
    -------
    function
        The decorated function wrapped within a pytest
        ``parametrize_fixture_doc`` mark
    """
    ...

def check_file_leaks(func) -> Callable:
    """
    Decorate a test function to check that we are not leaking file descriptors.
    """
    ...

@contextmanager
def file_leak_context() -> Iterator[None]:
    """
    ContextManager analogue to check_file_leaks.
    """
    ...

def async_mark():
    ...

def mark_array_manager_not_yet_implemented(request) -> None:
    ...

skip_array_manager_not_yet_implemented = ...
skip_array_manager_invalid_test = ...



"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable, Dict, Iterable, Mapping, TypeVar, Union

"""
Printing tools.
"""
EscapeChars = Union[Mapping[str, str], Iterable[str]]
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
def adjoin(space: int, *lists: list[str], **kwargs) -> str:
    """
    Glues together two sets of strings using the amount of space requested.
    The idea is to prettify.

    ----------
    space : int
        number of spaces for padding
    lists : str
        list of str which being joined
    strlen : callable
        function used to calculate the length of each str. Needed for unicode
        handling.
    justfunc : callable
        function used to justify str. Needed for unicode handling.
    """
    ...

def justify(texts: Iterable[str], max_len: int, mode: str = ...) -> list[str]:
    """
    Perform ljust, center, rjust against string or list-like
    """
    ...

def pprint_thing(thing: Any, _nest_lvl: int = ..., escape_chars: EscapeChars | None = ..., default_escapes: bool = ..., quote_strings: bool = ..., max_seq_items: int | None = ...) -> str:
    """
    This function is the sanctioned way of converting objects
    to a string representation and properly handles nested sequences.

    Parameters
    ----------
    thing : anything to be formatted
    _nest_lvl : internal use only. pprint_thing() is mutually-recursive
        with pprint_sequence, this argument is used to keep track of the
        current nesting level, and limit it.
    escape_chars : list or dict, optional
        Characters to escape. If a dict is passed the values are the
        replacements
    default_escapes : bool, default False
        Whether the input escape characters replaces or adds to the defaults
    max_seq_items : int or None, default None
        Pass through to other pretty printers to limit sequence printing

    Returns
    -------
    str
    """
    ...

def pprint_thing_encoded(object, encoding: str = ..., errors: str = ...) -> bytes:
    ...

def enable_data_resource_formatter(enable: bool) -> None:
    ...

def default_pprint(thing: Any, max_seq_items: int | None = ...) -> str:
    ...

def format_object_summary(obj, formatter: Callable, is_justify: bool = ..., name: str | None = ..., indent_for_name: bool = ..., line_break_each_value: bool = ...) -> str:
    """
    Return the formatted obj as a unicode string

    Parameters
    ----------
    obj : object
        must be iterable and support __getitem__
    formatter : callable
        string formatter for an element
    is_justify : bool
        should justify the display
    name : name, optional
        defaults to the class name of the obj
    indent_for_name : bool, default True
        Whether subsequent lines should be indented to
        align with the name.
    line_break_each_value : bool, default False
        If True, inserts a line break for each value of ``obj``.
        If False, only break lines when the a line of values gets wider
        than the display width.

        .. versionadded:: 0.25.0

    Returns
    -------
    summary string
    """
    ...

class PrettyDict(Dict[_KT, _VT]):
    """Dict extension to support abbreviated __repr__"""
    def __repr__(self) -> str:
        ...
    





"""
This type stub file was generated by pyright.
"""

import collections
import operator
import os
import re
import string
import warnings
import numpy as np
import pandas as pd
import pyarrow as pa
from __future__ import annotations
from datetime import datetime
from decimal import Decimal
from sys import byteorder
from typing import Callable, ContextManager, Counter, Iterable, TYPE_CHECKING
from pandas._config.localization import can_set_locale, get_locales, set_locale
from pandas._typing import Dtype
from pandas.compat import pa_version_under1p01
from pandas.core.dtypes.common import is_float_dtype, is_integer_dtype, is_sequence, is_unsigned_integer_dtype, pandas_dtype
from pandas import Categorical, CategoricalIndex, DataFrame, DatetimeIndex, Index, IntervalIndex, MultiIndex, PeriodIndex, RangeIndex, Series, TimedeltaIndex, bdate_range
from pandas._testing._io import close, network, round_trip_localpath, round_trip_pathlib, round_trip_pickle, write_to_compressed
from pandas._testing._random import randbool, rands, rands_array
from pandas._testing._warnings import assert_produces_warning, maybe_produces_warning
from pandas._testing.asserters import assert_almost_equal, assert_attr_equal, assert_categorical_equal, assert_class_equal, assert_contains_all, assert_copy, assert_datetime_array_equal, assert_dict_equal, assert_equal, assert_extension_array_equal, assert_frame_equal, assert_index_equal, assert_indexing_slices_equivalent, assert_interval_array_equal, assert_is_sorted, assert_is_valid_plot_return_object, assert_metadata_equivalent, assert_numpy_array_equal, assert_period_array_equal, assert_series_equal, assert_sp_array_equal, assert_timedelta_array_equal, raise_assert_detail
from pandas._testing.compat import get_dtype, get_obj
from pandas._testing.contexts import RNGContext, decompress_file, ensure_clean, ensure_clean_dir, ensure_safe_environment_variables, set_timezone, use_numexpr, with_csv_dialect
from pandas.core.api import Float64Index, Int64Index, NumericIndex, UInt64Index
from pandas.core.arrays import BaseMaskedArray, ExtensionArray, PandasArray
from pandas.core.arrays._mixins import NDArrayBackedExtensionArray
from pandas.core.construction import extract_array

if TYPE_CHECKING:
    ...
_N = ...
_K = ...
UNSIGNED_INT_NUMPY_DTYPES: list[Dtype] = ...
UNSIGNED_INT_EA_DTYPES: list[Dtype] = ...
SIGNED_INT_NUMPY_DTYPES: list[Dtype] = ...
SIGNED_INT_EA_DTYPES: list[Dtype] = ...
ALL_INT_NUMPY_DTYPES = ...
ALL_INT_EA_DTYPES = ...
FLOAT_NUMPY_DTYPES: list[Dtype] = ...
FLOAT_EA_DTYPES: list[Dtype] = ...
COMPLEX_DTYPES: list[Dtype] = ...
STRING_DTYPES: list[Dtype] = ...
DATETIME64_DTYPES: list[Dtype] = ...
TIMEDELTA64_DTYPES: list[Dtype] = ...
BOOL_DTYPES: list[Dtype] = ...
BYTES_DTYPES: list[Dtype] = ...
OBJECT_DTYPES: list[Dtype] = ...
ALL_REAL_NUMPY_DTYPES = ...
ALL_NUMPY_DTYPES = ...
NARROW_NP_DTYPES = ...
ENDIAN = ...
NULL_OBJECTS = ...
NP_NAT_OBJECTS = ...
if notpa_version_under1p01:
    UNSIGNED_INT_PYARROW_DTYPES = ...
    SIGNED_INT_PYARROW_DTYPES = ...
    ALL_INT_PYARROW_DTYPES = ...
    FLOAT_PYARROW_DTYPES = ...
    STRING_PYARROW_DTYPES = ...
    TIME_PYARROW_DTYPES = ...
    DATE_PYARROW_DTYPES = ...
    DATETIME_PYARROW_DTYPES = ...
    TIMEDELTA_PYARROW_DTYPES = ...
    BOOL_PYARROW_DTYPES = ...
    ALL_PYARROW_DTYPES = ...
EMPTY_STRING_PATTERN = ...
_testing_mode_warnings = ...
def set_testing_mode() -> None:
    ...

def reset_testing_mode() -> None:
    ...

def reset_display_options() -> None:
    """
    Reset the display options for printing and representing objects.
    """
    ...

def equalContents(arr1, arr2) -> bool:
    """
    Checks if the set of unique elements of arr1 and arr2 are equivalent.
    """
    ...

def box_expected(expected, box_cls, transpose=...): # -> ArrayLike | PandasArray | Index | Series | DataFrame | NDArray[Unknown]:
    """
    Helper function to wrap the expected output of a test in a given box_class.

    Parameters
    ----------
    expected : np.ndarray, Index, Series
    box_cls : {Index, Series, DataFrame}

    Returns
    -------
    subclass of box_cls
    """
    ...

def to_array(obj): # -> ArrayLike | NDArray[Unknown]:
    """
    Similar to pd.array, but does not cast numpy dtypes to nullable dtypes.
    """
    ...

def getCols(k) -> str:
    ...

def makeStringIndex(k=..., name=...) -> Index:
    ...

def makeCategoricalIndex(k=..., n=..., name=..., **kwargs) -> CategoricalIndex:
    """make a length k index or n categories"""
    ...

def makeIntervalIndex(k=..., name=..., **kwargs) -> IntervalIndex:
    """make a length k IntervalIndex"""
    ...

def makeBoolIndex(k=..., name=...) -> Index:
    ...

def makeNumericIndex(k=..., name=..., *, dtype) -> NumericIndex:
    ...

def makeIntIndex(k=..., name=...) -> Int64Index:
    ...

def makeUIntIndex(k=..., name=...) -> UInt64Index:
    ...

def makeRangeIndex(k=..., name=..., **kwargs) -> RangeIndex:
    ...

def makeFloatIndex(k=..., name=...) -> Float64Index:
    ...

def makeDateIndex(k: int = ..., freq=..., name=..., **kwargs) -> DatetimeIndex:
    ...

def makeTimedeltaIndex(k: int = ..., freq=..., name=..., **kwargs) -> TimedeltaIndex:
    ...

def makePeriodIndex(k: int = ..., name=..., **kwargs) -> PeriodIndex:
    ...

def makeMultiIndex(k=..., names=..., **kwargs): # -> tuple[Unknown, ...] | MultiIndex:
    ...

def index_subclass_makers_generator(): # -> Generator[Unknown, None, None]:
    ...

def all_timeseries_index_generator(k: int = ...) -> Iterable[Index]:
    """
    Generator which can be iterated over to get instances of all the classes
    which represent time-series.

    Parameters
    ----------
    k: length of each of the index instances
    """
    ...

def make_rand_series(name=..., dtype=...) -> Series:
    ...

def makeFloatSeries(name=...) -> Series:
    ...

def makeStringSeries(name=...) -> Series:
    ...

def makeObjectSeries(name=...) -> Series:
    ...

def getSeriesData() -> dict[str, Series]:
    ...

def makeTimeSeries(nper=..., freq=..., name=...) -> Series:
    ...

def makePeriodSeries(nper=..., name=...) -> Series:
    ...

def getTimeSeriesData(nper=..., freq=...) -> dict[str, Series]:
    ...

def getPeriodData(nper=...) -> dict[str, Series]:
    ...

def makeTimeDataFrame(nper=..., freq=...) -> DataFrame:
    ...

def makeDataFrame() -> DataFrame:
    ...

def getMixedTypeDict(): # -> tuple[Index, dict[str, Unknown]]:
    ...

def makeMixedDataFrame() -> DataFrame:
    ...

def makePeriodFrame(nper=...) -> DataFrame:
    ...

def makeCustomIndex(nentries, nlevels, prefix=..., names: bool | str | list[str] | None = ..., ndupe_l=..., idx_type=...) -> Index:
    """
    Create an index/multindex with given dimensions, levels, names, etc'

    nentries - number of entries in index
    nlevels - number of levels (> 1 produces multindex)
    prefix - a string prefix for labels
    names - (Optional), bool or list of strings. if True will use default
       names, if false will use no names, if a list is given, the name of
       each level in the index will be taken from the list.
    ndupe_l - (Optional), list of ints, the number of rows for which the
       label will repeated at the corresponding level, you can specify just
       the first few, the rest will use the default ndupe_l of 1.
       len(ndupe_l) <= nlevels.
    idx_type - "i"/"f"/"s"/"dt"/"p"/"td".
       If idx_type is not None, `idx_nlevels` must be 1.
       "i"/"f" creates an integer/float index,
       "s" creates a string
       "dt" create a datetime index.
       "td" create a datetime index.

        if unspecified, string labels will be generated.
    """
    ...

def makeCustomDataframe(nrows, ncols, c_idx_names=..., r_idx_names=..., c_idx_nlevels=..., r_idx_nlevels=..., data_gen_f=..., c_ndupe_l=..., r_ndupe_l=..., dtype=..., c_idx_type=..., r_idx_type=...) -> DataFrame:
    """
    Create a DataFrame using supplied parameters.

    Parameters
    ----------
    nrows,  ncols - number of data rows/cols
    c_idx_names, idx_names  - False/True/list of strings,  yields No names ,
            default names or uses the provided names for the levels of the
            corresponding index. You can provide a single string when
            c_idx_nlevels ==1.
    c_idx_nlevels - number of levels in columns index. > 1 will yield MultiIndex
    r_idx_nlevels - number of levels in rows index. > 1 will yield MultiIndex
    data_gen_f - a function f(row,col) which return the data value
            at that position, the default generator used yields values of the form
            "RxCy" based on position.
    c_ndupe_l, r_ndupe_l - list of integers, determines the number
            of duplicates for each label at a given level of the corresponding
            index. The default `None` value produces a multiplicity of 1 across
            all levels, i.e. a unique index. Will accept a partial list of length
            N < idx_nlevels, for just the first N levels. If ndupe doesn't divide
            nrows/ncol, the last label might have lower multiplicity.
    dtype - passed to the DataFrame constructor as is, in case you wish to
            have more control in conjunction with a custom `data_gen_f`
    r_idx_type, c_idx_type -  "i"/"f"/"s"/"dt"/"td".
        If idx_type is not None, `idx_nlevels` must be 1.
        "i"/"f" creates an integer/float index,
        "s" creates a string index
        "dt" create a datetime index.
        "td" create a timedelta index.

            if unspecified, string labels will be generated.

    Examples
    --------
    # 5 row, 3 columns, default names on both, single index on both axis
    >> makeCustomDataframe(5,3)

    # make the data a random int between 1 and 100
    >> mkdf(5,3,data_gen_f=lambda r,c:randint(1,100))

    # 2-level multiindex on rows with each label duplicated
    # twice on first level, default names on both axis, single
    # index on both axis
    >> a=makeCustomDataframe(5,3,r_idx_nlevels=2,r_ndupe_l=[2])

    # DatetimeIndex on row, index with unicode labels on columns
    # no names on either axis
    >> a=makeCustomDataframe(5,3,c_idx_names=False,r_idx_names=False,
                             r_idx_type="dt",c_idx_type="u")

    # 4-level multindex on rows with names provided, 2-level multindex
    # on columns with default labels and default names.
    >> a=makeCustomDataframe(5,3,r_idx_nlevels=4,
                             r_idx_names=["FEE","FIH","FOH","FUM"],
                             c_idx_nlevels=2)

    >> a=mkdf(5,3,r_idx_nlevels=2,c_idx_nlevels=4)
    """
    ...

def makeMissingDataframe(density=..., random_state=...) -> DataFrame:
    ...

class SubclassedSeries(Series):
    _metadata = ...


class SubclassedDataFrame(DataFrame):
    _metadata = ...


class SubclassedCategorical(Categorical):
    ...


def convert_rows_list_to_csv_str(rows_list: list[str]) -> str:
    """
    Convert list of CSV rows to single CSV-formatted string for current OS.

    This method is used for creating expected value of to_csv() method.

    Parameters
    ----------
    rows_list : List[str]
        Each element represents the row of csv.

    Returns
    -------
    str
        Expected output of to_csv() in current OS.
    """
    ...

def external_error_raised(expected_exception: type[Exception]) -> ContextManager:
    """
    Helper function to mark pytest.raises that have an external error message.

    Parameters
    ----------
    expected_exception : Exception
        Expected error to raise.

    Returns
    -------
    Callable
        Regular `pytest.raises` function with `match` equal to `None`.
    """
    ...

cython_table = ...
def get_cython_table_params(ndframe, func_names_and_expected): # -> list[Unknown]:
    """
    Combine frame, functions from com._cython_table
    keys and expected result.

    Parameters
    ----------
    ndframe : DataFrame or Series
    func_names_and_expected : Sequence of two items
        The first item is a name of a NDFrame method ('sum', 'prod') etc.
        The second item is the expected return value.

    Returns
    -------
    list
        List of three items (DataFrame, function, expected result)
    """
    ...

def get_op_from_name(op_name: str) -> Callable:
    """
    The operator function for a given op name.

    Parameters
    ----------
    op_name : str
        The op name, in form of "add" or "__add__".

    Returns
    -------
    function
        A function performing the operation.
    """
    ...

def getitem(x):
    ...

def setitem(x):
    ...

def loc(x):
    ...

def iloc(x):
    ...

def at(x):
    ...

def iat(x):
    ...

def shares_memory(left, right) -> bool:
    """
    Pandas-compat for np.shares_memory.
    """
    ...

__all__ = ["ALL_INT_EA_DTYPES", "ALL_INT_NUMPY_DTYPES", "ALL_NUMPY_DTYPES", "ALL_REAL_NUMPY_DTYPES", "all_timeseries_index_generator", "assert_almost_equal", "assert_attr_equal", "assert_categorical_equal", "assert_class_equal", "assert_contains_all", "assert_copy", "assert_datetime_array_equal", "assert_dict_equal", "assert_equal", "assert_extension_array_equal", "assert_frame_equal", "assert_index_equal", "assert_indexing_slices_equivalent", "assert_interval_array_equal", "assert_is_sorted", "assert_is_valid_plot_return_object", "assert_metadata_equivalent", "assert_numpy_array_equal", "assert_period_array_equal", "assert_produces_warning", "assert_series_equal", "assert_sp_array_equal", "assert_timedelta_array_equal", "at", "BOOL_DTYPES", "box_expected", "BYTES_DTYPES", "can_set_locale", "close", "COMPLEX_DTYPES", "convert_rows_list_to_csv_str", "DATETIME64_DTYPES", "decompress_file", "EMPTY_STRING_PATTERN", "ENDIAN", "ensure_clean", "ensure_clean_dir", "ensure_safe_environment_variables", "equalContents", "external_error_raised", "FLOAT_EA_DTYPES", "FLOAT_NUMPY_DTYPES", "getCols", "get_cython_table_params", "get_dtype", "getitem", "get_locales", "getMixedTypeDict", "get_obj", "get_op_from_name", "getPeriodData", "getSeriesData", "getTimeSeriesData", "iat", "iloc", "index_subclass_makers_generator", "loc", "makeBoolIndex", "makeCategoricalIndex", "makeCustomDataframe", "makeCustomIndex", "makeDataFrame", "makeDateIndex", "makeFloatIndex", "makeFloatSeries", "makeIntervalIndex", "makeIntIndex", "makeMissingDataframe", "makeMixedDataFrame", "makeMultiIndex", "makeNumericIndex", "makeObjectSeries", "makePeriodFrame", "makePeriodIndex", "makePeriodSeries", "make_rand_series", "makeRangeIndex", "makeStringIndex", "makeStringSeries", "makeTimeDataFrame", "makeTimedeltaIndex", "makeTimeSeries", "makeUIntIndex", "maybe_produces_warning", "NARROW_NP_DTYPES", "network", "NP_NAT_OBJECTS", "NULL_OBJECTS", "OBJECT_DTYPES", "raise_assert_detail", "randbool", "rands", "reset_display_options", "reset_testing_mode", "RNGContext", "round_trip_localpath", "round_trip_pathlib", "round_trip_pickle", "setitem", "set_locale", "set_testing_mode", "set_timezone", "shares_memory", "SIGNED_INT_EA_DTYPES", "SIGNED_INT_NUMPY_DTYPES", "STRING_DTYPES", "SubclassedCategorical", "SubclassedDataFrame", "SubclassedSeries", "TIMEDELTA64_DTYPES", "to_array", "UNSIGNED_INT_EA_DTYPES", "UNSIGNED_INT_NUMPY_DTYPES", "use_numexpr", "with_csv_dialect", "write_to_compressed"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import tzinfo
from typing import Hashable, Literal, TYPE_CHECKING
from pandas._libs import lib
from pandas._libs.tslibs import BaseOffset
from pandas._typing import Dtype, IntervalClosedType, IntervalLeftRight, npt
from pandas.util._decorators import doc
from pandas.core.arrays.datetimes import DatetimeArray
from pandas.core.indexes.base import Index
from pandas.core.indexes.datetimelike import DatetimeTimedeltaMixin
from pandas.core.indexes.extension import inherit_names
from pandas import DataFrame, Float64Index, PeriodIndex, TimedeltaIndex

if TYPE_CHECKING:
    ...
@inherit_names(DatetimeArray._field_ops + [method for method in DatetimeArray._datetimelike_methods if method not in ("tz_localize", "tz_convert", "strftime")], DatetimeArray, wrap=True)
@inherit_names(["is_normalized", "_resolution_obj"], DatetimeArray, cache=True)
@inherit_names(["tz", "tzinfo", "dtype", "to_pydatetime", "_format_native_types", "date", "time", "timetz", "std"] + DatetimeArray._bool_ops, DatetimeArray)
class DatetimeIndex(DatetimeTimedeltaMixin):
    """
    Immutable ndarray-like of datetime64 data.

    Represented internally as int64, and which can be boxed to Timestamp objects
    that are subclasses of datetime and carry metadata.

    Parameters
    ----------
    data : array-like (1-dimensional)
        Datetime-like data to construct index with.
    freq : str or pandas offset object, optional
        One of pandas date offset strings or corresponding objects. The string
        'infer' can be passed in order to set the frequency of the index as the
        inferred frequency upon creation.
    tz : pytz.timezone or dateutil.tz.tzfile or datetime.tzinfo or str
        Set the Timezone of the data.
    normalize : bool, default False
        Normalize start/end dates to midnight before generating date range.
    closed : {'left', 'right'}, optional
        Set whether to include `start` and `end` that are on the
        boundary. The default includes boundary points on either end.
    ambiguous : 'infer', bool-ndarray, 'NaT', default 'raise'
        When clocks moved backward due to DST, ambiguous times may arise.
        For example in Central European Time (UTC+01), when going from 03:00
        DST to 02:00 non-DST, 02:30:00 local time occurs both at 00:30:00 UTC
        and at 01:30:00 UTC. In such a situation, the `ambiguous` parameter
        dictates how ambiguous times should be handled.

        - 'infer' will attempt to infer fall dst-transition hours based on
          order
        - bool-ndarray where True signifies a DST time, False signifies a
          non-DST time (note that this flag is only applicable for ambiguous
          times)
        - 'NaT' will return NaT where there are ambiguous times
        - 'raise' will raise an AmbiguousTimeError if there are ambiguous times.
    dayfirst : bool, default False
        If True, parse dates in `data` with the day first order.
    yearfirst : bool, default False
        If True parse dates in `data` with the year first order.
    dtype : numpy.dtype or DatetimeTZDtype or str, default None
        Note that the only NumPy dtype allowed is ‘datetime64[ns]’.
    copy : bool, default False
        Make a copy of input ndarray.
    name : label, default None
        Name to be stored in the index.

    Attributes
    ----------
    year
    month
    day
    hour
    minute
    second
    microsecond
    nanosecond
    date
    time
    timetz
    dayofyear
    day_of_year
    weekofyear
    week
    dayofweek
    day_of_week
    weekday
    quarter
    tz
    freq
    freqstr
    is_month_start
    is_month_end
    is_quarter_start
    is_quarter_end
    is_year_start
    is_year_end
    is_leap_year
    inferred_freq

    Methods
    -------
    normalize
    strftime
    snap
    tz_convert
    tz_localize
    round
    floor
    ceil
    to_period
    to_perioddelta
    to_pydatetime
    to_series
    to_frame
    month_name
    day_name
    mean
    std

    See Also
    --------
    Index : The base pandas Index type.
    TimedeltaIndex : Index of timedelta64 data.
    PeriodIndex : Index of Period data.
    to_datetime : Convert argument to datetime.
    date_range : Create a fixed-frequency DatetimeIndex.

    Notes
    -----
    To learn more about the frequency strings, please see `this link
    <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`__.
    """
    _typ = ...
    _data_cls = DatetimeArray
    _supports_partial_string_indexing = ...
    _data: DatetimeArray
    inferred_freq: str | None
    tz: tzinfo | None
    @doc(DatetimeArray.strftime)
    def strftime(self, date_format) -> Index:
        ...
    
    @doc(DatetimeArray.tz_convert)
    def tz_convert(self, tz) -> DatetimeIndex:
        ...
    
    @doc(DatetimeArray.tz_localize)
    def tz_localize(self, tz, ambiguous=..., nonexistent=...) -> DatetimeIndex:
        ...
    
    @doc(DatetimeArray.to_period)
    def to_period(self, freq=...) -> PeriodIndex:
        ...
    
    @doc(DatetimeArray.to_perioddelta)
    def to_perioddelta(self, freq) -> TimedeltaIndex:
        ...
    
    @doc(DatetimeArray.to_julian_date)
    def to_julian_date(self) -> Float64Index:
        ...
    
    @doc(DatetimeArray.isocalendar)
    def isocalendar(self) -> DataFrame:
        ...
    
    def __new__(cls, data=..., freq: str | BaseOffset | lib.NoDefault = ..., tz=..., normalize: bool = ..., closed=..., ambiguous=..., dayfirst: bool = ..., yearfirst: bool = ..., dtype: Dtype | None = ..., copy: bool = ..., name: Hashable = ...) -> DatetimeIndex:
        ...
    
    def __reduce__(self): # -> tuple[(cls: Unknown, d: Unknown) -> Unknown, tuple[Type[Self@DatetimeIndex], dict[str, Unknown]], None]:
        ...
    
    def union_many(self, others): # -> ABCMultiIndex | Self@DatetimeIndex | Index | None:
        """
        A bit of a hack to accelerate unioning a collection of indexes.
        """
        ...
    
    def to_series(self, keep_tz=..., index=..., name=...): # -> Series:
        """
        Create a Series with both index and values equal to the index keys.

        Useful with map for returning an indexer based on an index.

        Parameters
        ----------
        keep_tz : optional, defaults True
            Return the data keeping the timezone.

            If keep_tz is True:

              If the timezone is not set, the resulting
              Series will have a datetime64[ns] dtype.

              Otherwise the Series will have an datetime64[ns, tz] dtype; the
              tz will be preserved.

            If keep_tz is False:

              Series will have a datetime64[ns] dtype. TZ aware
              objects will have the tz removed.

            .. versionchanged:: 1.0.0
                The default value is now True.  In a future version,
                this keyword will be removed entirely.  Stop passing the
                argument to obtain the future behavior and silence the warning.

        index : Index, optional
            Index of resulting Series. If None, defaults to original index.
        name : str, optional
            Name of resulting Series. If None, defaults to name of original
            index.

        Returns
        -------
        Series
        """
        ...
    
    def snap(self, freq=...) -> DatetimeIndex:
        """
        Snap time stamps to nearest occurring frequency.

        Returns
        -------
        DatetimeIndex
        """
        ...
    
    def get_loc(self, key, method=..., tolerance=...): # -> slice | ndarray[Any, _dtype] | int | ndarray[Unknown, Unknown]:
        """
        Get integer location for requested label

        Returns
        -------
        loc : int
        """
        ...
    
    def slice_indexer(self, start=..., end=..., step=..., kind=...): # -> NDArray[intp] | slice:
        """
        Return indexer for specified label slice.
        Index.slice_indexer, customized to handle time slicing.

        In addition to functionality provided by Index.slice_indexer, does the
        following:

        - if both `start` and `end` are instances of `datetime.time`, it
          invokes `indexer_between_time`
        - if `start` and `end` are both either string or None perform
          value-based selection in non-monotonic cases.

        """
        ...
    
    @property
    def inferred_type(self) -> str:
        ...
    
    def indexer_at_time(self, time, asof: bool = ...) -> npt.NDArray[np.intp]:
        """
        Return index locations of values at particular time of day.

        Parameters
        ----------
        time : datetime.time or str
            Time passed in either as object (datetime.time) or as string in
            appropriate format ("%H:%M", "%H%M", "%I:%M%p", "%I%M%p",
            "%H:%M:%S", "%H%M%S", "%I:%M:%S%p", "%I%M%S%p").

        Returns
        -------
        np.ndarray[np.intp]

        See Also
        --------
        indexer_between_time : Get index locations of values between particular
            times of day.
        DataFrame.at_time : Select values at particular time of day.
        """
        ...
    
    def indexer_between_time(self, start_time, end_time, include_start: bool = ..., include_end: bool = ...) -> npt.NDArray[np.intp]:
        """
        Return index locations of values between particular times of day.

        Parameters
        ----------
        start_time, end_time : datetime.time, str
            Time passed either as object (datetime.time) or as string in
            appropriate format ("%H:%M", "%H%M", "%I:%M%p", "%I%M%p",
            "%H:%M:%S", "%H%M%S", "%I:%M:%S%p","%I%M%S%p").
        include_start : bool, default True
        include_end : bool, default True

        Returns
        -------
        np.ndarray[np.intp]

        See Also
        --------
        indexer_at_time : Get index locations of values at particular time of day.
        DataFrame.between_time : Select values between particular times of day.
        """
        ...
    


def date_range(start=..., end=..., periods=..., freq=..., tz=..., normalize: bool = ..., name: Hashable = ..., closed: Literal["left", "right"] | None | lib.NoDefault = ..., inclusive: IntervalClosedType | None = ..., **kwargs) -> DatetimeIndex:
    """
    Return a fixed frequency DatetimeIndex.

    Returns the range of equally spaced time points (where the difference between any
    two adjacent points is specified by the given frequency) such that they all
    satisfy `start <[=] x <[=] end`, where the first one and the last one are, resp.,
    the first and last time points in that range that fall on the boundary of ``freq``
    (if given as a frequency string) or that are valid for ``freq`` (if given as a
    :class:`pandas.tseries.offsets.DateOffset`). (If exactly one of ``start``,
    ``end``, or ``freq`` is *not* specified, this missing parameter can be computed
    given ``periods``, the number of timesteps in the range. See the note below.)

    Parameters
    ----------
    start : str or datetime-like, optional
        Left bound for generating dates.
    end : str or datetime-like, optional
        Right bound for generating dates.
    periods : int, optional
        Number of periods to generate.
    freq : str or DateOffset, default 'D'
        Frequency strings can have multiples, e.g. '5H'. See
        :ref:`here <timeseries.offset_aliases>` for a list of
        frequency aliases.
    tz : str or tzinfo, optional
        Time zone name for returning localized DatetimeIndex, for example
        'Asia/Hong_Kong'. By default, the resulting DatetimeIndex is
        timezone-naive.
    normalize : bool, default False
        Normalize start/end dates to midnight before generating date range.
    name : str, default None
        Name of the resulting DatetimeIndex.
    closed : {None, 'left', 'right'}, optional
        Make the interval closed with respect to the given frequency to
        the 'left', 'right', or both sides (None, the default).

        .. deprecated:: 1.4.0
           Argument `closed` has been deprecated to standardize boundary inputs.
           Use `inclusive` instead, to set each bound as closed or open.
    inclusive : {"both", "neither", "left", "right"}, default "both"
        Include boundaries; Whether to set each bound as closed or open.

        .. versionadded:: 1.4.0
    **kwargs
        For compatibility. Has no effect on the result.

    Returns
    -------
    rng : DatetimeIndex

    See Also
    --------
    DatetimeIndex : An immutable container for datetimes.
    timedelta_range : Return a fixed frequency TimedeltaIndex.
    period_range : Return a fixed frequency PeriodIndex.
    interval_range : Return a fixed frequency IntervalIndex.

    Notes
    -----
    Of the four parameters ``start``, ``end``, ``periods``, and ``freq``,
    exactly three must be specified. If ``freq`` is omitted, the resulting
    ``DatetimeIndex`` will have ``periods`` linearly spaced elements between
    ``start`` and ``end`` (closed on both sides).

    To learn more about the frequency strings, please see `this link
    <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`__.

    Examples
    --------
    **Specifying the values**

    The next four examples generate the same `DatetimeIndex`, but vary
    the combination of `start`, `end` and `periods`.

    Specify `start` and `end`, with the default daily frequency.

    >>> pd.date_range(start='1/1/2018', end='1/08/2018')
    DatetimeIndex(['2018-01-01', '2018-01-02', '2018-01-03', '2018-01-04',
                   '2018-01-05', '2018-01-06', '2018-01-07', '2018-01-08'],
                  dtype='datetime64[ns]', freq='D')

    Specify `start` and `periods`, the number of periods (days).

    >>> pd.date_range(start='1/1/2018', periods=8)
    DatetimeIndex(['2018-01-01', '2018-01-02', '2018-01-03', '2018-01-04',
                   '2018-01-05', '2018-01-06', '2018-01-07', '2018-01-08'],
                  dtype='datetime64[ns]', freq='D')

    Specify `end` and `periods`, the number of periods (days).

    >>> pd.date_range(end='1/1/2018', periods=8)
    DatetimeIndex(['2017-12-25', '2017-12-26', '2017-12-27', '2017-12-28',
                   '2017-12-29', '2017-12-30', '2017-12-31', '2018-01-01'],
                  dtype='datetime64[ns]', freq='D')

    Specify `start`, `end`, and `periods`; the frequency is generated
    automatically (linearly spaced).

    >>> pd.date_range(start='2018-04-24', end='2018-04-27', periods=3)
    DatetimeIndex(['2018-04-24 00:00:00', '2018-04-25 12:00:00',
                   '2018-04-27 00:00:00'],
                  dtype='datetime64[ns]', freq=None)

    **Other Parameters**

    Changed the `freq` (frequency) to ``'M'`` (month end frequency).

    >>> pd.date_range(start='1/1/2018', periods=5, freq='M')
    DatetimeIndex(['2018-01-31', '2018-02-28', '2018-03-31', '2018-04-30',
                   '2018-05-31'],
                  dtype='datetime64[ns]', freq='M')

    Multiples are allowed

    >>> pd.date_range(start='1/1/2018', periods=5, freq='3M')
    DatetimeIndex(['2018-01-31', '2018-04-30', '2018-07-31', '2018-10-31',
                   '2019-01-31'],
                  dtype='datetime64[ns]', freq='3M')

    `freq` can also be specified as an Offset object.

    >>> pd.date_range(start='1/1/2018', periods=5, freq=pd.offsets.MonthEnd(3))
    DatetimeIndex(['2018-01-31', '2018-04-30', '2018-07-31', '2018-10-31',
                   '2019-01-31'],
                  dtype='datetime64[ns]', freq='3M')

    Specify `tz` to set the timezone.

    >>> pd.date_range(start='1/1/2018', periods=5, tz='Asia/Tokyo')
    DatetimeIndex(['2018-01-01 00:00:00+09:00', '2018-01-02 00:00:00+09:00',
                   '2018-01-03 00:00:00+09:00', '2018-01-04 00:00:00+09:00',
                   '2018-01-05 00:00:00+09:00'],
                  dtype='datetime64[ns, Asia/Tokyo]', freq='D')

    `inclusive` controls whether to include `start` and `end` that are on the
    boundary. The default, "both", includes boundary points on either end.

    >>> pd.date_range(start='2017-01-01', end='2017-01-04', inclusive="both")
    DatetimeIndex(['2017-01-01', '2017-01-02', '2017-01-03', '2017-01-04'],
                  dtype='datetime64[ns]', freq='D')

    Use ``inclusive='left'`` to exclude `end` if it falls on the boundary.

    >>> pd.date_range(start='2017-01-01', end='2017-01-04', inclusive='left')
    DatetimeIndex(['2017-01-01', '2017-01-02', '2017-01-03'],
                  dtype='datetime64[ns]', freq='D')

    Use ``inclusive='right'`` to exclude `start` if it falls on the boundary, and
    similarly ``inclusive='neither'`` will exclude both `start` and `end`.

    >>> pd.date_range(start='2017-01-01', end='2017-01-04', inclusive='right')
    DatetimeIndex(['2017-01-02', '2017-01-03', '2017-01-04'],
                  dtype='datetime64[ns]', freq='D')
    """
    ...

def bdate_range(start=..., end=..., periods: int | None = ..., freq=..., tz=..., normalize: bool = ..., name: Hashable = ..., weekmask=..., holidays=..., closed: IntervalLeftRight | lib.NoDefault | None = ..., inclusive: IntervalClosedType | None = ..., **kwargs) -> DatetimeIndex:
    """
    Return a fixed frequency DatetimeIndex with business day as the default.

    Parameters
    ----------
    start : str or datetime-like, default None
        Left bound for generating dates.
    end : str or datetime-like, default None
        Right bound for generating dates.
    periods : int, default None
        Number of periods to generate.
    freq : str or DateOffset, default 'B' (business daily)
        Frequency strings can have multiples, e.g. '5H'.
    tz : str or None
        Time zone name for returning localized DatetimeIndex, for example
        Asia/Beijing.
    normalize : bool, default False
        Normalize start/end dates to midnight before generating date range.
    name : str, default None
        Name of the resulting DatetimeIndex.
    weekmask : str or None, default None
        Weekmask of valid business days, passed to ``numpy.busdaycalendar``,
        only used when custom frequency strings are passed.  The default
        value None is equivalent to 'Mon Tue Wed Thu Fri'.
    holidays : list-like or None, default None
        Dates to exclude from the set of valid business days, passed to
        ``numpy.busdaycalendar``, only used when custom frequency strings
        are passed.
    closed : str, default None
        Make the interval closed with respect to the given frequency to
        the 'left', 'right', or both sides (None).

        .. deprecated:: 1.4.0
           Argument `closed` has been deprecated to standardize boundary inputs.
           Use `inclusive` instead, to set each bound as closed or open.
    inclusive : {"both", "neither", "left", "right"}, default "both"
        Include boundaries; Whether to set each bound as closed or open.

        .. versionadded:: 1.4.0
    **kwargs
        For compatibility. Has no effect on the result.

    Returns
    -------
    DatetimeIndex

    Notes
    -----
    Of the four parameters: ``start``, ``end``, ``periods``, and ``freq``,
    exactly three must be specified.  Specifying ``freq`` is a requirement
    for ``bdate_range``.  Use ``date_range`` if specifying ``freq`` is not
    desired.

    To learn more about the frequency strings, please see `this link
    <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`__.

    Examples
    --------
    Note how the two weekend days are skipped in the result.

    >>> pd.bdate_range(start='1/1/2018', end='1/08/2018')
    DatetimeIndex(['2018-01-01', '2018-01-02', '2018-01-03', '2018-01-04',
               '2018-01-05', '2018-01-08'],
              dtype='datetime64[ns]', freq='B')
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
import numpy.typing as npt
from datetime import datetime, timedelta, tzinfo
from typing import Any, Callable, Dict, Hashable, Iterator, List, Literal, Mapping, Optional, Protocol, Sequence, TYPE_CHECKING, Tuple, Type as type_t, TypeVar, Union

if TYPE_CHECKING:
    NumpyValueArrayLike = Union[npt._ScalarLike_co, npt.ArrayLike]
    NumpySorter = Optional[npt._ArrayLikeInt_co]
else:
    ...
HashableT = TypeVar("HashableT", bound=Hashable)
ArrayLike = Union["ExtensionArray", np.ndarray]
AnyArrayLike = Union[ArrayLike, "Index", "Series"]
PythonScalar = Union[str, float, bool]
DatetimeLikeScalar = Union["Period", "Timestamp", "Timedelta"]
PandasScalar = Union["Period", "Timestamp", "Timedelta", "Interval"]
Scalar = Union[PythonScalar, PandasScalar, np.datetime64, np.timedelta64, datetime]
IntStrT = TypeVar("IntStrT", int, str)
TimestampConvertibleTypes = Union["Timestamp", datetime, np.datetime64, np.int64, float, str]
TimedeltaConvertibleTypes = Union["Timedelta", timedelta, np.timedelta64, np.int64, float, str]
Timezone = Union[str, tzinfo]
NDFrameT = TypeVar("NDFrameT", bound="NDFrame")
NumpyIndexT = TypeVar("NumpyIndexT", np.ndarray, "Index")
Axis = Union[str, int]
IndexLabel = Union[Hashable, Sequence[Hashable]]
Level = Hashable
Shape = Tuple[int, ...]
Suffixes = Tuple[Optional[str], Optional[str]]
Ordered = Optional[bool]
JSONSerializable = Optional[Union[PythonScalar, List, Dict]]
Frequency = Union[str, "BaseOffset"]
Axes = Union[AnyArrayLike, List, range]
RandomState = Union[int, ArrayLike, np.random.Generator, np.random.BitGenerator, np.random.RandomState,]
NpDtype = Union[str, np.dtype, type_t[Union[str, complex, bool, object]]]
Dtype = Union["ExtensionDtype", NpDtype]
AstypeArg = Union["ExtensionDtype", "npt.DTypeLike"]
DtypeArg = Union[Dtype, Dict[Hashable, Dtype]]
DtypeObj = Union[np.dtype, "ExtensionDtype"]
ConvertersArg = Dict[Hashable, Callable[[Dtype], Dtype]]
ParseDatesArg = Union[bool, List[Hashable], List[List[Hashable]], Dict[Hashable, List[Hashable]]]
Renamer = Union[Mapping[Any, Hashable], Callable[[Any], Hashable]]
T = TypeVar("T")
FuncType = Callable[..., Any]
F = TypeVar("F", bound=FuncType)
ValueKeyFunc = Optional[Callable[["Series"], Union["Series", AnyArrayLike]]]
IndexKeyFunc = Optional[Callable[["Index"], Union["Index", AnyArrayLike]]]
AggFuncTypeBase = Union[Callable, str]
AggFuncTypeDict = Dict[Hashable, Union[AggFuncTypeBase, List[AggFuncTypeBase]]]
AggFuncType = Union[AggFuncTypeBase, List[AggFuncTypeBase], AggFuncTypeDict,]
AggObjType = Union["Series", "DataFrame", "GroupBy", "SeriesGroupBy", "DataFrameGroupBy", "BaseWindow", "Resampler",]
PythonFuncType = Callable[[Any], Any]
AnyStr_cov = TypeVar("AnyStr_cov", str, bytes, covariant=True)
AnyStr_con = TypeVar("AnyStr_con", str, bytes, contravariant=True)
class BaseBuffer(Protocol):
    @property
    def mode(self) -> str:
        ...
    
    def fileno(self) -> int:
        ...
    
    def seek(self, __offset: int, __whence: int = ...) -> int:
        ...
    
    def seekable(self) -> bool:
        ...
    
    def tell(self) -> int:
        ...
    


class ReadBuffer(BaseBuffer, Protocol[AnyStr_cov]):
    def read(self, __n: int | None = ...) -> AnyStr_cov:
        ...
    


class WriteBuffer(BaseBuffer, Protocol[AnyStr_con]):
    def write(self, __b: AnyStr_con) -> Any:
        ...
    
    def flush(self) -> Any:
        ...
    


class ReadPickleBuffer(ReadBuffer[bytes], Protocol):
    def readline(self) -> AnyStr_cov:
        ...
    


class WriteExcelBuffer(WriteBuffer[bytes], Protocol):
    def truncate(self, size: int | None = ...) -> int:
        ...
    


class ReadCsvBuffer(ReadBuffer[AnyStr_cov], Protocol):
    def __iter__(self) -> Iterator[AnyStr_cov]:
        ...
    
    def readline(self) -> AnyStr_cov:
        ...
    
    @property
    def closed(self) -> bool:
        ...
    


FilePath = Union[str, "PathLike[str]"]
StorageOptions = Optional[Dict[str, Any]]
CompressionDict = Dict[str, Any]
CompressionOptions = Optional[Union[Literal["infer", "gzip", "bz2", "zip", "xz", "zstd", "tar"], CompressionDict]]
FormattersType = Union[List[Callable], Tuple[Callable, ...], Mapping[Union[str, int], Callable]]
ColspaceType = Mapping[Hashable, Union[str, int]]
FloatFormatType = Union[str, Callable, "EngFormatter"]
ColspaceArgType = Union[str, int, Sequence[Union[str, int]], Mapping[Hashable, Union[str, int]]]
FillnaOptions = Literal["backfill", "bfill", "ffill", "pad"]
Manager = Union["ArrayManager", "SingleArrayManager", "BlockManager", "SingleBlockManager"]
SingleManager = Union["SingleArrayManager", "SingleBlockManager"]
Manager2D = Union["ArrayManager", "BlockManager"]
ScalarIndexer = Union[int, np.integer]
SequenceIndexer = Union[slice, List[int], np.ndarray]
PositionalIndexer = Union[ScalarIndexer, SequenceIndexer]
PositionalIndexerTuple = Tuple[PositionalIndexer, PositionalIndexer]
PositionalIndexer2D = Union[PositionalIndexer, PositionalIndexerTuple]
if TYPE_CHECKING:
    TakeIndexer = Union[Sequence[int], Sequence[np.integer], npt.NDArray[np.integer]]
else:
    ...
IgnoreRaise = Literal["ignore", "raise"]
WindowingRankType = Literal["average", "min", "max"]
CSVEngine = Literal["c", "python", "pyarrow", "python-fwf"]
XMLParsers = Literal["lxml", "etree"]
IntervalLeftRight = Literal["left", "right"]
IntervalClosedType = Union[IntervalLeftRight, Literal["both", "neither"]]
DatetimeNaTType = Union[datetime, "NaTType"]
DateTimeErrorChoices = Union[IgnoreRaise, Literal["coerce"]]
SortKind = Literal["quicksort", "mergesort", "heapsort", "stable"]
NaPosition = Literal["first", "last"]
QuantileInterpolation = Literal["linear", "lower", "higher", "midpoint", "nearest"]
PlottingOrientation = Literal["horizontal", "vertical"]



"""
This type stub file was generated by pyright.
"""

from pandas.compat import is_platform_windows

"""
Hypothesis data generator helpers.
"""
OPTIONAL_INTS = ...
OPTIONAL_FLOATS = ...
OPTIONAL_TEXT = ...
OPTIONAL_DICTS = ...
OPTIONAL_LISTS = ...
OPTIONAL_ONE_OF_ALL = ...
if is_platform_windows():
    DATETIME_NO_TZ = ...
else:
    DATETIME_NO_TZ = ...
DATETIME_JAN_1_1900_OPTIONAL_TZ = ...
DATETIME_IN_PD_TIMESTAMP_RANGE_NO_TZ = ...
INT_NEG_999_TO_POS_999 = ...
YQM_OFFSET = ...



"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

def inner_join(left: np.ndarray, right: np.ndarray, max_groups: int) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

def left_outer_join(left: np.ndarray, right: np.ndarray, max_groups: int, sort: bool = ...) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

def full_outer_join(left: np.ndarray, right: np.ndarray, max_groups: int) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

def ffill_indexer(indexer: np.ndarray) -> npt.NDArray[np.intp]:
    ...

def left_join_indexer_unique(left: np.ndarray, right: np.ndarray) -> npt.NDArray[np.intp]:
    ...

def left_join_indexer(left: np.ndarray, right: np.ndarray) -> tuple[np.ndarray, npt.NDArray[np.intp], npt.NDArray[np.intp],]:
    ...

def inner_join_indexer(left: np.ndarray, right: np.ndarray) -> tuple[np.ndarray, npt.NDArray[np.intp], npt.NDArray[np.intp],]:
    ...

def outer_join_indexer(left: np.ndarray, right: np.ndarray) -> tuple[np.ndarray, npt.NDArray[np.intp], npt.NDArray[np.intp],]:
    ...

def asof_join_backward_on_X_by_Y(left_values: np.ndarray, right_values: np.ndarray, left_by_values: np.ndarray, right_by_values: np.ndarray, allow_exact_matches: bool = ..., tolerance: np.number | float | None = ..., use_hashtable: bool = ...) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

def asof_join_forward_on_X_by_Y(left_values: np.ndarray, right_values: np.ndarray, left_by_values: np.ndarray, right_by_values: np.ndarray, allow_exact_matches: bool = ..., tolerance: np.number | float | None = ..., use_hashtable: bool = ...) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...

def asof_join_nearest_on_X_by_Y(left_values: np.ndarray, right_values: np.ndarray, left_by_values: np.ndarray, right_by_values: np.ndarray, allow_exact_matches: bool = ..., tolerance: np.number | float | None = ..., use_hashtable: bool = ...) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np

"""
Missing data handling for arithmetic operations.

In particular, pandas conventions regarding division by zero differ
from numpy in the following ways:
    1) np.array([-1, 0, 1], dtype=dtype1) // np.array([0, 0, 0], dtype=dtype2)
       gives [nan, nan, nan] for most dtype combinations, and [0, 0, 0] for
       the remaining pairs
       (the remaining being dtype1==dtype2==intN and dtype==dtype2==uintN).

       pandas convention is to return [-inf, nan, inf] for all dtype
       combinations.

       Note: the numpy behavior described here is py3-specific.

    2) np.array([-1, 0, 1], dtype=dtype1) % np.array([0, 0, 0], dtype=dtype2)
       gives precisely the same results as the // operation.

       pandas convention is to return [nan, nan, nan] for all dtype
       combinations.

    3) divmod behavior consistent with 1) and 2).
"""
def mask_zero_div_zero(x, y, result: np.ndarray) -> np.ndarray:
    """
    Set results of  0 // 0 to np.nan, regardless of the dtypes
    of the numerator or the denominator.

    Parameters
    ----------
    x : ndarray
    y : ndarray
    result : ndarray

    Returns
    -------
    ndarray
        The filled result.

    Examples
    --------
    >>> x = np.array([1, 0, -1], dtype=np.int64)
    >>> x
    array([ 1,  0, -1])
    >>> y = 0       # int 0; numpy behavior is different with float
    >>> result = x // y
    >>> result      # raw numpy result does not fill division by zero
    array([0, 0, 0])
    >>> mask_zero_div_zero(x, y, result)
    array([ inf,  nan, -inf])
    """
    ...

def dispatch_fill_zeros(op, left, right, result): # -> tuple[ndarray[Unknown, Unknown], Unknown] | ndarray[Unknown, Unknown]:
    """
    Call _fill_zeros with the appropriate fill value depending on the operation,
    with special logic for divmod and rdivmod.

    Parameters
    ----------
    op : function (operator.add, operator.div, ...)
    left : object (np.ndarray for non-reversed ops)
    right : object (np.ndarray for reversed ops)
    result : ndarray

    Returns
    -------
    result : np.ndarray

    Notes
    -----
    For divmod and rdivmod, the `result` parameter and returned `result`
    is a 2-tuple of ndarray objects.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Hashable, TYPE_CHECKING, final
from pandas._typing import ArrayLike, NDFrameT, npt
from pandas.util._decorators import cache_readonly
from pandas.core.arrays import Categorical
from pandas.core.groupby import ops
from pandas.core.indexes.api import Index
from pandas.core.generic import NDFrame

"""
Provide user facing operators for doing the split part of the
split-apply-combine paradigm.
"""
if TYPE_CHECKING:
    ...
class Grouper:
    """
    A Grouper allows the user to specify a groupby instruction for an object.

    This specification will select a column via the key parameter, or if the
    level and/or axis parameters are given, a level of the index of the target
    object.

    If `axis` and/or `level` are passed as keywords to both `Grouper` and
    `groupby`, the values passed to `Grouper` take precedence.

    Parameters
    ----------
    key : str, defaults to None
        Groupby key, which selects the grouping column of the target.
    level : name/number, defaults to None
        The level for the target index.
    freq : str / frequency object, defaults to None
        This will groupby the specified frequency if the target selection
        (via key or level) is a datetime-like object. For full specification
        of available frequencies, please see `here
        <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`_.
    axis : str, int, defaults to 0
        Number/name of the axis.
    sort : bool, default to False
        Whether to sort the resulting labels.
    closed : {'left' or 'right'}
        Closed end of interval. Only when `freq` parameter is passed.
    label : {'left' or 'right'}
        Interval boundary to use for labeling.
        Only when `freq` parameter is passed.
    convention : {'start', 'end', 'e', 's'}
        If grouper is PeriodIndex and `freq` parameter is passed.
    base : int, default 0
        Only when `freq` parameter is passed.
        For frequencies that evenly subdivide 1 day, the "origin" of the
        aggregated intervals. For example, for '5min' frequency, base could
        range from 0 through 4. Defaults to 0.

        .. deprecated:: 1.1.0
            The new arguments that you should use are 'offset' or 'origin'.

    loffset : str, DateOffset, timedelta object
        Only when `freq` parameter is passed.

        .. deprecated:: 1.1.0
            loffset is only working for ``.resample(...)`` and not for
            Grouper (:issue:`28302`).
            However, loffset is also deprecated for ``.resample(...)``
            See: :class:`DataFrame.resample`

    origin : Timestamp or str, default 'start_day'
        The timestamp on which to adjust the grouping. The timezone of origin must
        match the timezone of the index.
        If string, must be one of the following:

        - 'epoch': `origin` is 1970-01-01
        - 'start': `origin` is the first value of the timeseries
        - 'start_day': `origin` is the first day at midnight of the timeseries

        .. versionadded:: 1.1.0

        - 'end': `origin` is the last value of the timeseries
        - 'end_day': `origin` is the ceiling midnight of the last day

        .. versionadded:: 1.3.0

    offset : Timedelta or str, default is None
        An offset timedelta added to the origin.

        .. versionadded:: 1.1.0

    dropna : bool, default True
        If True, and if group keys contain NA values, NA values together with
        row/column will be dropped. If False, NA values will also be treated as
        the key in groups.

        .. versionadded:: 1.2.0

    Returns
    -------
    A specification for a groupby instruction

    Examples
    --------
    Syntactic sugar for ``df.groupby('A')``

    >>> df = pd.DataFrame(
    ...     {
    ...         "Animal": ["Falcon", "Parrot", "Falcon", "Falcon", "Parrot"],
    ...         "Speed": [100, 5, 200, 300, 15],
    ...     }
    ... )
    >>> df
       Animal  Speed
    0  Falcon    100
    1  Parrot      5
    2  Falcon    200
    3  Falcon    300
    4  Parrot     15
    >>> df.groupby(pd.Grouper(key="Animal")).mean()
            Speed
    Animal
    Falcon  200.0
    Parrot   10.0

    Specify a resample operation on the column 'Publish date'

    >>> df = pd.DataFrame(
    ...    {
    ...        "Publish date": [
    ...             pd.Timestamp("2000-01-02"),
    ...             pd.Timestamp("2000-01-02"),
    ...             pd.Timestamp("2000-01-09"),
    ...             pd.Timestamp("2000-01-16")
    ...         ],
    ...         "ID": [0, 1, 2, 3],
    ...         "Price": [10, 20, 30, 40]
    ...     }
    ... )
    >>> df
      Publish date  ID  Price
    0   2000-01-02   0     10
    1   2000-01-02   1     20
    2   2000-01-09   2     30
    3   2000-01-16   3     40
    >>> df.groupby(pd.Grouper(key="Publish date", freq="1W")).mean()
                   ID  Price
    Publish date
    2000-01-02    0.5   15.0
    2000-01-09    2.0   30.0
    2000-01-16    3.0   40.0

    If you want to adjust the start of the bins based on a fixed timestamp:

    >>> start, end = '2000-10-01 23:30:00', '2000-10-02 00:30:00'
    >>> rng = pd.date_range(start, end, freq='7min')
    >>> ts = pd.Series(np.arange(len(rng)) * 3, index=rng)
    >>> ts
    2000-10-01 23:30:00     0
    2000-10-01 23:37:00     3
    2000-10-01 23:44:00     6
    2000-10-01 23:51:00     9
    2000-10-01 23:58:00    12
    2000-10-02 00:05:00    15
    2000-10-02 00:12:00    18
    2000-10-02 00:19:00    21
    2000-10-02 00:26:00    24
    Freq: 7T, dtype: int64

    >>> ts.groupby(pd.Grouper(freq='17min')).sum()
    2000-10-01 23:14:00     0
    2000-10-01 23:31:00     9
    2000-10-01 23:48:00    21
    2000-10-02 00:05:00    54
    2000-10-02 00:22:00    24
    Freq: 17T, dtype: int64

    >>> ts.groupby(pd.Grouper(freq='17min', origin='epoch')).sum()
    2000-10-01 23:18:00     0
    2000-10-01 23:35:00    18
    2000-10-01 23:52:00    27
    2000-10-02 00:09:00    39
    2000-10-02 00:26:00    24
    Freq: 17T, dtype: int64

    >>> ts.groupby(pd.Grouper(freq='17min', origin='2000-01-01')).sum()
    2000-10-01 23:24:00     3
    2000-10-01 23:41:00    15
    2000-10-01 23:58:00    45
    2000-10-02 00:15:00    45
    Freq: 17T, dtype: int64

    If you want to adjust the start of the bins with an `offset` Timedelta, the two
    following lines are equivalent:

    >>> ts.groupby(pd.Grouper(freq='17min', origin='start')).sum()
    2000-10-01 23:30:00     9
    2000-10-01 23:47:00    21
    2000-10-02 00:04:00    54
    2000-10-02 00:21:00    24
    Freq: 17T, dtype: int64

    >>> ts.groupby(pd.Grouper(freq='17min', offset='23h30min')).sum()
    2000-10-01 23:30:00     9
    2000-10-01 23:47:00    21
    2000-10-02 00:04:00    54
    2000-10-02 00:21:00    24
    Freq: 17T, dtype: int64

    To replace the use of the deprecated `base` argument, you can now use `offset`,
    in this example it is equivalent to have `base=2`:

    >>> ts.groupby(pd.Grouper(freq='17min', offset='2min')).sum()
    2000-10-01 23:16:00     0
    2000-10-01 23:33:00     9
    2000-10-01 23:50:00    36
    2000-10-02 00:07:00    39
    2000-10-02 00:24:00    24
    Freq: 17T, dtype: int64
    """
    axis: int
    sort: bool
    dropna: bool
    _gpr_index: Index | None
    _grouper: Index | None
    _attributes: tuple[str, ...] = ...
    def __new__(cls, *args, **kwargs): # -> TimeGrouper | Self@Grouper:
        ...
    
    def __init__(self, key=..., level=..., freq=..., axis: int = ..., sort: bool = ..., dropna: bool = ...) -> None:
        ...
    
    @final
    @property
    def ax(self) -> Index:
        ...
    
    @final
    @property
    def groups(self): # -> dict[Hashable, ndarray[Unknown, Unknown]]:
        ...
    
    @final
    def __repr__(self) -> str:
        ...
    


@final
class Grouping:
    """
    Holds the grouping information for a single key

    Parameters
    ----------
    index : Index
    grouper :
    obj : DataFrame or Series
    name : Label
    level :
    observed : bool, default False
        If we are a Categorical, use the observed values
    in_axis : if the Grouping is a column in self.obj and hence among
        Groupby.exclusions list

    Returns
    -------
    **Attributes**:
      * indices : dict of {group -> index_list}
      * codes : ndarray, group codes
      * group_index : unique groups
      * groups : dict of {group -> label_list}
    """
    _codes: npt.NDArray[np.signedinteger] | None = ...
    _group_index: Index | None = ...
    _passed_categorical: bool
    _all_grouper: Categorical | None
    _index: Index
    def __init__(self, index: Index, grouper=..., obj: NDFrame | None = ..., level=..., sort: bool = ..., observed: bool = ..., in_axis: bool = ..., dropna: bool = ...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    def __iter__(self): # -> Iterator[Hashable]:
        ...
    
    @cache_readonly
    def name(self) -> Hashable:
        ...
    
    @property
    def ngroups(self) -> int:
        ...
    
    @cache_readonly
    def indices(self) -> dict[Hashable, npt.NDArray[np.intp]]:
        ...
    
    @property
    def codes(self) -> npt.NDArray[np.signedinteger]:
        ...
    
    @cache_readonly
    def group_arraylike(self) -> ArrayLike:
        """
        Analogous to result_index, but holding an ArrayLike to ensure
        we can retain ExtensionDtypes.
        """
        ...
    
    @cache_readonly
    def result_index(self) -> Index:
        ...
    
    @cache_readonly
    def group_index(self) -> Index:
        ...
    
    @cache_readonly
    def groups(self) -> dict[Hashable, np.ndarray]:
        ...
    


def get_grouper(obj: NDFrameT, key=..., axis: int = ..., level=..., sort: bool = ..., observed: bool = ..., mutated: bool = ..., validate: bool = ..., dropna: bool = ...) -> tuple[ops.BaseGrouper, frozenset[Hashable], NDFrameT]:
    """
    Create and return a BaseGrouper, which is an internal
    mapping of how to create the grouper indexers.
    This may be composed of multiple Grouping objects, indicating
    multiple groupers

    Groupers are ultimately index mappings. They can originate as:
    index mappings, keys to columns, functions, or Groupers

    Groupers enable local references to axis,level,sort, while
    the passed in axis, level, and sort are 'global'.

    This routine tries to figure out what the passing in references
    are and then creates a Grouping for each one, combined into
    a BaseGrouper.

    If observed & we have a categorical grouper, only show the observed
    values.

    If validate, then check for key/level overlaps.

    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from contextlib import contextmanager
from typing import Iterator, TYPE_CHECKING, overload
from pandas._typing import DtypeArg, IndexLabel
from pandas.core.api import DataFrame
from pandas.core.base import PandasObject
from sqlalchemy import Table

"""
Collection of query wrappers / abstractions to both facilitate data
retrieval and to reduce dependency on DB-specific API.
"""
if TYPE_CHECKING:
    ...
def execute(sql, con, params=...):
    """
    Execute the given SQL query using the provided connection object.

    Parameters
    ----------
    sql : string
        SQL query to be executed.
    con : SQLAlchemy connectable(engine/connection) or sqlite3 connection
        Using SQLAlchemy makes it possible to use any DB supported by the
        library.
        If a DBAPI2 object, only sqlite3 is supported.
    params : list or tuple, optional, default: None
        List of parameters to pass to execute method.

    Returns
    -------
    Results Iterable
    """
    ...

@overload
def read_sql_table(table_name, con, schema=..., index_col: str | list[str] | None = ..., coerce_float=..., parse_dates: list[str] | dict[str, str] | None = ..., columns: list[str] | None = ..., chunksize: None = ...) -> DataFrame:
    ...

@overload
def read_sql_table(table_name, con, schema=..., index_col: str | list[str] | None = ..., coerce_float=..., parse_dates: list[str] | dict[str, str] | None = ..., columns: list[str] | None = ..., chunksize: int = ...) -> Iterator[DataFrame]:
    ...

def read_sql_table(table_name: str, con, schema: str | None = ..., index_col: str | list[str] | None = ..., coerce_float: bool = ..., parse_dates: list[str] | dict[str, str] | None = ..., columns: list[str] | None = ..., chunksize: int | None = ...) -> DataFrame | Iterator[DataFrame]:
    """
    Read SQL database table into a DataFrame.

    Given a table name and a SQLAlchemy connectable, returns a DataFrame.
    This function does not support DBAPI connections.

    Parameters
    ----------
    table_name : str
        Name of SQL table in database.
    con : SQLAlchemy connectable or str
        A database URI could be provided as str.
        SQLite DBAPI connection mode not supported.
    schema : str, default None
        Name of SQL schema in database to query (if database flavor
        supports this). Uses default schema if None (default).
    index_col : str or list of str, optional, default: None
        Column(s) to set as index(MultiIndex).
    coerce_float : bool, default True
        Attempts to convert values of non-string, non-numeric objects (like
        decimal.Decimal) to floating point. Can result in loss of Precision.
    parse_dates : list or dict, default None
        - List of column names to parse as dates.
        - Dict of ``{column_name: format string}`` where format string is
          strftime compatible in case of parsing string times or is one of
          (D, s, ns, ms, us) in case of parsing integer timestamps.
        - Dict of ``{column_name: arg dict}``, where the arg dict corresponds
          to the keyword arguments of :func:`pandas.to_datetime`
          Especially useful with databases without native Datetime support,
          such as SQLite.
    columns : list, default None
        List of column names to select from SQL table.
    chunksize : int, default None
        If specified, returns an iterator where `chunksize` is the number of
        rows to include in each chunk.

    Returns
    -------
    DataFrame or Iterator[DataFrame]
        A SQL table is returned as two-dimensional data structure with labeled
        axes.

    See Also
    --------
    read_sql_query : Read SQL query into a DataFrame.
    read_sql : Read SQL query or database table into a DataFrame.

    Notes
    -----
    Any datetime values with time zone information will be converted to UTC.

    Examples
    --------
    >>> pd.read_sql_table('table_name', 'postgres:///db_name')  # doctest:+SKIP
    """
    ...

@overload
def read_sql_query(sql, con, index_col: str | list[str] | None = ..., coerce_float=..., params: list[str] | dict[str, str] | None = ..., parse_dates: list[str] | dict[str, str] | None = ..., chunksize: None = ..., dtype: DtypeArg | None = ...) -> DataFrame:
    ...

@overload
def read_sql_query(sql, con, index_col: str | list[str] | None = ..., coerce_float=..., params: list[str] | dict[str, str] | None = ..., parse_dates: list[str] | dict[str, str] | None = ..., chunksize: int = ..., dtype: DtypeArg | None = ...) -> Iterator[DataFrame]:
    ...

def read_sql_query(sql, con, index_col: str | list[str] | None = ..., coerce_float: bool = ..., params: list[str] | dict[str, str] | None = ..., parse_dates: list[str] | dict[str, str] | None = ..., chunksize: int | None = ..., dtype: DtypeArg | None = ...) -> DataFrame | Iterator[DataFrame]:
    """
    Read SQL query into a DataFrame.

    Returns a DataFrame corresponding to the result set of the query
    string. Optionally provide an `index_col` parameter to use one of the
    columns as the index, otherwise default integer index will be used.

    Parameters
    ----------
    sql : str SQL query or SQLAlchemy Selectable (select or text object)
        SQL query to be executed.
    con : SQLAlchemy connectable, str, or sqlite3 connection
        Using SQLAlchemy makes it possible to use any DB supported by that
        library. If a DBAPI2 object, only sqlite3 is supported.
    index_col : str or list of str, optional, default: None
        Column(s) to set as index(MultiIndex).
    coerce_float : bool, default True
        Attempts to convert values of non-string, non-numeric objects (like
        decimal.Decimal) to floating point. Useful for SQL result sets.
    params : list, tuple or dict, optional, default: None
        List of parameters to pass to execute method.  The syntax used
        to pass parameters is database driver dependent. Check your
        database driver documentation for which of the five syntax styles,
        described in PEP 249's paramstyle, is supported.
        Eg. for psycopg2, uses %(name)s so use params={'name' : 'value'}.
    parse_dates : list or dict, default: None
        - List of column names to parse as dates.
        - Dict of ``{column_name: format string}`` where format string is
          strftime compatible in case of parsing string times, or is one of
          (D, s, ns, ms, us) in case of parsing integer timestamps.
        - Dict of ``{column_name: arg dict}``, where the arg dict corresponds
          to the keyword arguments of :func:`pandas.to_datetime`
          Especially useful with databases without native Datetime support,
          such as SQLite.
    chunksize : int, default None
        If specified, return an iterator where `chunksize` is the number of
        rows to include in each chunk.
    dtype : Type name or dict of columns
        Data type for data or columns. E.g. np.float64 or
        {‘a’: np.float64, ‘b’: np.int32, ‘c’: ‘Int64’}.

        .. versionadded:: 1.3.0

    Returns
    -------
    DataFrame or Iterator[DataFrame]

    See Also
    --------
    read_sql_table : Read SQL database table into a DataFrame.
    read_sql : Read SQL query or database table into a DataFrame.

    Notes
    -----
    Any datetime values with time zone information parsed via the `parse_dates`
    parameter will be converted to UTC.
    """
    ...

@overload
def read_sql(sql, con, index_col: str | list[str] | None = ..., coerce_float=..., params=..., parse_dates=..., columns: list[str] = ..., chunksize: None = ...) -> DataFrame:
    ...

@overload
def read_sql(sql, con, index_col: str | list[str] | None = ..., coerce_float=..., params=..., parse_dates=..., columns: list[str] = ..., chunksize: int = ...) -> Iterator[DataFrame]:
    ...

def read_sql(sql, con, index_col: str | list[str] | None = ..., coerce_float: bool = ..., params=..., parse_dates=..., columns: list[str] | None = ..., chunksize: int | None = ...) -> DataFrame | Iterator[DataFrame]:
    """
    Read SQL query or database table into a DataFrame.

    This function is a convenience wrapper around ``read_sql_table`` and
    ``read_sql_query`` (for backward compatibility). It will delegate
    to the specific function depending on the provided input. A SQL query
    will be routed to ``read_sql_query``, while a database table name will
    be routed to ``read_sql_table``. Note that the delegated function might
    have more specific notes about their functionality not listed here.

    Parameters
    ----------
    sql : str or SQLAlchemy Selectable (select or text object)
        SQL query to be executed or a table name.
    con : SQLAlchemy connectable, str, or sqlite3 connection
        Using SQLAlchemy makes it possible to use any DB supported by that
        library. If a DBAPI2 object, only sqlite3 is supported. The user is responsible
        for engine disposal and connection closure for the SQLAlchemy connectable; str
        connections are closed automatically. See
        `here <https://docs.sqlalchemy.org/en/13/core/connections.html>`_.
    index_col : str or list of str, optional, default: None
        Column(s) to set as index(MultiIndex).
    coerce_float : bool, default True
        Attempts to convert values of non-string, non-numeric objects (like
        decimal.Decimal) to floating point, useful for SQL result sets.
    params : list, tuple or dict, optional, default: None
        List of parameters to pass to execute method.  The syntax used
        to pass parameters is database driver dependent. Check your
        database driver documentation for which of the five syntax styles,
        described in PEP 249's paramstyle, is supported.
        Eg. for psycopg2, uses %(name)s so use params={'name' : 'value'}.
    parse_dates : list or dict, default: None
        - List of column names to parse as dates.
        - Dict of ``{column_name: format string}`` where format string is
          strftime compatible in case of parsing string times, or is one of
          (D, s, ns, ms, us) in case of parsing integer timestamps.
        - Dict of ``{column_name: arg dict}``, where the arg dict corresponds
          to the keyword arguments of :func:`pandas.to_datetime`
          Especially useful with databases without native Datetime support,
          such as SQLite.
    columns : list, default: None
        List of column names to select from SQL table (only used when reading
        a table).
    chunksize : int, default None
        If specified, return an iterator where `chunksize` is the
        number of rows to include in each chunk.

    Returns
    -------
    DataFrame or Iterator[DataFrame]

    See Also
    --------
    read_sql_table : Read SQL database table into a DataFrame.
    read_sql_query : Read SQL query into a DataFrame.

    Examples
    --------
    Read data from SQL via either a SQL query or a SQL tablename.
    When using a SQLite database only SQL queries are accepted,
    providing only the SQL tablename will result in an error.

    >>> from sqlite3 import connect
    >>> conn = connect(':memory:')
    >>> df = pd.DataFrame(data=[[0, '10/11/12'], [1, '12/11/10']],
    ...                   columns=['int_column', 'date_column'])
    >>> df.to_sql('test_data', conn)
    2

    >>> pd.read_sql('SELECT int_column, date_column FROM test_data', conn)
       int_column date_column
    0           0    10/11/12
    1           1    12/11/10

    >>> pd.read_sql('test_data', 'postgres:///db_name')  # doctest:+SKIP

    Apply date parsing to columns through the ``parse_dates`` argument

    >>> pd.read_sql('SELECT int_column, date_column FROM test_data',
    ...             conn,
    ...             parse_dates=["date_column"])
       int_column date_column
    0           0  2012-10-11
    1           1  2010-12-11

    The ``parse_dates`` argument calls ``pd.to_datetime`` on the provided columns.
    Custom argument values for applying ``pd.to_datetime`` on a column are specified
    via a dictionary format:
    1. Ignore errors while parsing the values of "date_column"

    >>> pd.read_sql('SELECT int_column, date_column FROM test_data',
    ...             conn,
    ...             parse_dates={"date_column": {"errors": "ignore"}})
       int_column date_column
    0           0  2012-10-11
    1           1  2010-12-11

    2. Apply a dayfirst date parsing order on the values of "date_column"

    >>> pd.read_sql('SELECT int_column, date_column FROM test_data',
    ...             conn,
    ...             parse_dates={"date_column": {"dayfirst": True}})
       int_column date_column
    0           0  2012-11-10
    1           1  2010-11-12

    3. Apply custom formatting when date parsing the values of "date_column"

    >>> pd.read_sql('SELECT int_column, date_column FROM test_data',
    ...             conn,
    ...             parse_dates={"date_column": {"format": "%d/%m/%y"}})
       int_column date_column
    0           0  2012-11-10
    1           1  2010-11-12
    """
    ...

def to_sql(frame, name: str, con, schema: str | None = ..., if_exists: str = ..., index: bool = ..., index_label: IndexLabel = ..., chunksize: int | None = ..., dtype: DtypeArg | None = ..., method: str | None = ..., engine: str = ..., **engine_kwargs) -> int | None:
    """
    Write records stored in a DataFrame to a SQL database.

    Parameters
    ----------
    frame : DataFrame, Series
    name : str
        Name of SQL table.
    con : SQLAlchemy connectable(engine/connection) or database string URI
        or sqlite3 DBAPI2 connection
        Using SQLAlchemy makes it possible to use any DB supported by that
        library.
        If a DBAPI2 object, only sqlite3 is supported.
    schema : str, optional
        Name of SQL schema in database to write to (if database flavor
        supports this). If None, use default schema (default).
    if_exists : {'fail', 'replace', 'append'}, default 'fail'
        - fail: If table exists, do nothing.
        - replace: If table exists, drop it, recreate it, and insert data.
        - append: If table exists, insert data. Create if does not exist.
    index : bool, default True
        Write DataFrame index as a column.
    index_label : str or sequence, optional
        Column label for index column(s). If None is given (default) and
        `index` is True, then the index names are used.
        A sequence should be given if the DataFrame uses MultiIndex.
    chunksize : int, optional
        Specify the number of rows in each batch to be written at a time.
        By default, all rows will be written at once.
    dtype : dict or scalar, optional
        Specifying the datatype for columns. If a dictionary is used, the
        keys should be the column names and the values should be the
        SQLAlchemy types or strings for the sqlite3 fallback mode. If a
        scalar is provided, it will be applied to all columns.
    method : {None, 'multi', callable}, optional
        Controls the SQL insertion clause used:

        - None : Uses standard SQL ``INSERT`` clause (one per row).
        - ``'multi'``: Pass multiple values in a single ``INSERT`` clause.
        - callable with signature ``(pd_table, conn, keys, data_iter) -> int | None``.

        Details and a sample callable implementation can be found in the
        section :ref:`insert method <io.sql.method>`.
    engine : {'auto', 'sqlalchemy'}, default 'auto'
        SQL engine library to use. If 'auto', then the option
        ``io.sql.engine`` is used. The default ``io.sql.engine``
        behavior is 'sqlalchemy'

        .. versionadded:: 1.3.0

    **engine_kwargs
        Any additional kwargs are passed to the engine.

    Returns
    -------
    None or int
        Number of rows affected by to_sql. None is returned if the callable
        passed into ``method`` does not return an integer number of rows.

        .. versionadded:: 1.4.0

    Notes
    -----
    The returned rows affected is the sum of the ``rowcount`` attribute of ``sqlite3.Cursor``
    or SQLAlchemy connectable. The returned value may not reflect the exact number of written
    rows as stipulated in the
    `sqlite3 <https://docs.python.org/3/library/sqlite3.html#sqlite3.Cursor.rowcount>`__ or
    `SQLAlchemy <https://docs.sqlalchemy.org/en/14/core/connections.html#sqlalchemy.engine.BaseCursorResult.rowcount>`__
    """
    ...

def has_table(table_name: str, con, schema: str | None = ...) -> bool:
    """
    Check if DataBase has named table.

    Parameters
    ----------
    table_name: string
        Name of SQL table.
    con: SQLAlchemy connectable(engine/connection) or sqlite3 DBAPI2 connection
        Using SQLAlchemy makes it possible to use any DB supported by that
        library.
        If a DBAPI2 object, only sqlite3 is supported.
    schema : string, default None
        Name of SQL schema in database to write to (if database flavor supports
        this). If None, use default schema (default).

    Returns
    -------
    boolean
    """
    ...

table_exists = ...
def pandasSQL_builder(con, schema: str | None = ...) -> SQLDatabase | SQLiteDatabase:
    """
    Convenience function to return the correct PandasSQL subclass based on the
    provided parameters.
    """
    ...

class SQLTable(PandasObject):
    """
    For mapping Pandas tables to SQL tables.
    Uses fact that table is reflected by SQLAlchemy to
    do better type conversions.
    Also holds various flags needed to avoid having to
    pass them between functions all the time.
    """
    def __init__(self, name: str, pandas_sql_engine, frame=..., index: bool | str | list[str] | None = ..., if_exists: str = ..., prefix: str = ..., index_label=..., schema=..., keys=..., dtype: DtypeArg | None = ...) -> None:
        ...
    
    def exists(self):
        ...
    
    def sql_schema(self) -> str:
        ...
    
    def create(self) -> None:
        ...
    
    def insert_data(self) -> tuple[list[str], list[np.ndarray]]:
        ...
    
    def insert(self, chunksize: int | None = ..., method: str | None = ...) -> int | None:
        ...
    
    def read(self, coerce_float: bool = ..., parse_dates=..., columns=..., chunksize=...) -> DataFrame | Iterator[DataFrame]:
        ...
    


class PandasSQL(PandasObject):
    """
    Subclasses Should define read_sql and to_sql.
    """
    def read_sql(self, *args, **kwargs):
        ...
    
    def to_sql(self, frame, name, if_exists: str = ..., index: bool = ..., index_label=..., schema=..., chunksize=..., dtype: DtypeArg | None = ..., method=...) -> int | None:
        ...
    


class BaseEngine:
    def insert_records(self, table: SQLTable, con, frame, name, index=..., schema=..., chunksize=..., method=..., **engine_kwargs) -> int | None:
        """
        Inserts data into already-prepared table
        """
        ...
    


class SQLAlchemyEngine(BaseEngine):
    def __init__(self) -> None:
        ...
    
    def insert_records(self, table: SQLTable, con, frame, name, index=..., schema=..., chunksize=..., method=..., **engine_kwargs) -> int | None:
        ...
    


def get_engine(engine: str) -> BaseEngine:
    """return our implementation"""
    ...

class SQLDatabase(PandasSQL):
    """
    This class enables conversion between DataFrame and SQL databases
    using SQLAlchemy to handle DataBase abstraction.

    Parameters
    ----------
    engine : SQLAlchemy connectable
        Connectable to connect with the database. Using SQLAlchemy makes it
        possible to use any DB supported by that library.
    schema : string, default None
        Name of SQL schema in database to write to (if database flavor
        supports this). If None, use default schema (default).

    """
    def __init__(self, engine, schema: str | None = ...) -> None:
        ...
    
    @contextmanager
    def run_transaction(self): # -> Generator[Connection | Unknown, None, None]:
        ...
    
    def execute(self, *args, **kwargs):
        """Simple passthrough to SQLAlchemy connectable"""
        ...
    
    def read_table(self, table_name: str, index_col: str | list[str] | None = ..., coerce_float: bool = ..., parse_dates=..., columns=..., schema: str | None = ..., chunksize: int | None = ...) -> DataFrame | Iterator[DataFrame]:
        """
        Read SQL database table into a DataFrame.

        Parameters
        ----------
        table_name : str
            Name of SQL table in database.
        index_col : string, optional, default: None
            Column to set as index.
        coerce_float : bool, default True
            Attempts to convert values of non-string, non-numeric objects
            (like decimal.Decimal) to floating point. This can result in
            loss of precision.
        parse_dates : list or dict, default: None
            - List of column names to parse as dates.
            - Dict of ``{column_name: format string}`` where format string is
              strftime compatible in case of parsing string times, or is one of
              (D, s, ns, ms, us) in case of parsing integer timestamps.
            - Dict of ``{column_name: arg}``, where the arg corresponds
              to the keyword arguments of :func:`pandas.to_datetime`.
              Especially useful with databases without native Datetime support,
              such as SQLite.
        columns : list, default: None
            List of column names to select from SQL table.
        schema : string, default None
            Name of SQL schema in database to query (if database flavor
            supports this).  If specified, this overwrites the default
            schema of the SQL database object.
        chunksize : int, default None
            If specified, return an iterator where `chunksize` is the number
            of rows to include in each chunk.

        Returns
        -------
        DataFrame

        See Also
        --------
        pandas.read_sql_table
        SQLDatabase.read_query

        """
        ...
    
    def read_query(self, sql: str, index_col: str | list[str] | None = ..., coerce_float: bool = ..., parse_dates=..., params=..., chunksize: int | None = ..., dtype: DtypeArg | None = ...) -> DataFrame | Iterator[DataFrame]:
        """
        Read SQL query into a DataFrame.

        Parameters
        ----------
        sql : str
            SQL query to be executed.
        index_col : string, optional, default: None
            Column name to use as index for the returned DataFrame object.
        coerce_float : bool, default True
            Attempt to convert values of non-string, non-numeric objects (like
            decimal.Decimal) to floating point, useful for SQL result sets.
        params : list, tuple or dict, optional, default: None
            List of parameters to pass to execute method.  The syntax used
            to pass parameters is database driver dependent. Check your
            database driver documentation for which of the five syntax styles,
            described in PEP 249's paramstyle, is supported.
            Eg. for psycopg2, uses %(name)s so use params={'name' : 'value'}
        parse_dates : list or dict, default: None
            - List of column names to parse as dates.
            - Dict of ``{column_name: format string}`` where format string is
              strftime compatible in case of parsing string times, or is one of
              (D, s, ns, ms, us) in case of parsing integer timestamps.
            - Dict of ``{column_name: arg dict}``, where the arg dict
              corresponds to the keyword arguments of
              :func:`pandas.to_datetime` Especially useful with databases
              without native Datetime support, such as SQLite.
        chunksize : int, default None
            If specified, return an iterator where `chunksize` is the number
            of rows to include in each chunk.
        dtype : Type name or dict of columns
            Data type for data or columns. E.g. np.float64 or
            {‘a’: np.float64, ‘b’: np.int32, ‘c’: ‘Int64’}

            .. versionadded:: 1.3.0

        Returns
        -------
        DataFrame

        See Also
        --------
        read_sql_table : Read SQL database table into a DataFrame.
        read_sql

        """
        ...
    
    read_sql = ...
    def prep_table(self, frame, name, if_exists=..., index=..., index_label=..., schema=..., dtype: DtypeArg | None = ...) -> SQLTable:
        """
        Prepares table in the database for data insertion. Creates it if needed, etc.
        """
        ...
    
    def check_case_sensitive(self, name, schema) -> None:
        """
        Checks table name for issues with case-sensitivity.
        Method is called after data is inserted.
        """
        ...
    
    def to_sql(self, frame, name, if_exists: str = ..., index: bool = ..., index_label=..., schema=..., chunksize=..., dtype: DtypeArg | None = ..., method=..., engine=..., **engine_kwargs) -> int | None:
        """
        Write records stored in a DataFrame to a SQL database.

        Parameters
        ----------
        frame : DataFrame
        name : string
            Name of SQL table.
        if_exists : {'fail', 'replace', 'append'}, default 'fail'
            - fail: If table exists, do nothing.
            - replace: If table exists, drop it, recreate it, and insert data.
            - append: If table exists, insert data. Create if does not exist.
        index : boolean, default True
            Write DataFrame index as a column.
        index_label : string or sequence, default None
            Column label for index column(s). If None is given (default) and
            `index` is True, then the index names are used.
            A sequence should be given if the DataFrame uses MultiIndex.
        schema : string, default None
            Name of SQL schema in database to write to (if database flavor
            supports this). If specified, this overwrites the default
            schema of the SQLDatabase object.
        chunksize : int, default None
            If not None, then rows will be written in batches of this size at a
            time.  If None, all rows will be written at once.
        dtype : single type or dict of column name to SQL type, default None
            Optional specifying the datatype for columns. The SQL type should
            be a SQLAlchemy type. If all columns are of the same type, one
            single value can be used.
        method : {None', 'multi', callable}, default None
            Controls the SQL insertion clause used:

            * None : Uses standard SQL ``INSERT`` clause (one per row).
            * 'multi': Pass multiple values in a single ``INSERT`` clause.
            * callable with signature ``(pd_table, conn, keys, data_iter)``.

            Details and a sample callable implementation can be found in the
            section :ref:`insert method <io.sql.method>`.
        engine : {'auto', 'sqlalchemy'}, default 'auto'
            SQL engine library to use. If 'auto', then the option
            ``io.sql.engine`` is used. The default ``io.sql.engine``
            behavior is 'sqlalchemy'

            .. versionadded:: 1.3.0

        **engine_kwargs
            Any additional kwargs are passed to the engine.
        """
        ...
    
    @property
    def tables(self): # -> Any:
        ...
    
    def has_table(self, name: str, schema: str | None = ...):
        ...
    
    def get_table(self, table_name: str, schema: str | None = ...) -> Table:
        ...
    
    def drop_table(self, table_name: str, schema: str | None = ...) -> None:
        ...
    


_SQL_TYPES = ...
class SQLiteTable(SQLTable):
    """
    Patch the SQLTable for fallback support.
    Instead of a table variable just use the Create Table statement.
    """
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    def sql_schema(self) -> str:
        ...
    
    def insert_statement(self, *, num_rows: int) -> str:
        ...
    


class SQLiteDatabase(PandasSQL):
    """
    Version of SQLDatabase to support SQLite connections (fallback without
    SQLAlchemy). This should only be used internally.

    Parameters
    ----------
    con : sqlite connection object

    """
    def __init__(self, con) -> None:
        ...
    
    @contextmanager
    def run_transaction(self): # -> Generator[Unknown, None, None]:
        ...
    
    def execute(self, *args, **kwargs):
        ...
    
    def read_query(self, sql, index_col=..., coerce_float: bool = ..., params=..., parse_dates=..., chunksize: int | None = ..., dtype: DtypeArg | None = ...) -> DataFrame | Iterator[DataFrame]:
        ...
    
    def to_sql(self, frame, name, if_exists: str = ..., index: bool = ..., index_label=..., schema=..., chunksize=..., dtype: DtypeArg | None = ..., method=..., **kwargs) -> int | None:
        """
        Write records stored in a DataFrame to a SQL database.

        Parameters
        ----------
        frame: DataFrame
        name: string
            Name of SQL table.
        if_exists: {'fail', 'replace', 'append'}, default 'fail'
            fail: If table exists, do nothing.
            replace: If table exists, drop it, recreate it, and insert data.
            append: If table exists, insert data. Create if it does not exist.
        index : bool, default True
            Write DataFrame index as a column
        index_label : string or sequence, default None
            Column label for index column(s). If None is given (default) and
            `index` is True, then the index names are used.
            A sequence should be given if the DataFrame uses MultiIndex.
        schema : string, default None
            Ignored parameter included for compatibility with SQLAlchemy
            version of ``to_sql``.
        chunksize : int, default None
            If not None, then rows will be written in batches of this
            size at a time. If None, all rows will be written at once.
        dtype : single type or dict of column name to SQL type, default None
            Optional specifying the datatype for columns. The SQL type should
            be a string. If all columns are of the same type, one single value
            can be used.
        method : {None, 'multi', callable}, default None
            Controls the SQL insertion clause used:

            * None : Uses standard SQL ``INSERT`` clause (one per row).
            * 'multi': Pass multiple values in a single ``INSERT`` clause.
            * callable with signature ``(pd_table, conn, keys, data_iter)``.

            Details and a sample callable implementation can be found in the
            section :ref:`insert method <io.sql.method>`.
        """
        ...
    
    def has_table(self, name: str, schema: str | None = ...) -> bool:
        ...
    
    def get_table(self, table_name: str, schema: str | None = ...) -> None:
        ...
    
    def drop_table(self, name: str, schema: str | None = ...) -> None:
        ...
    


def get_schema(frame, name: str, keys=..., con=..., dtype: DtypeArg | None = ..., schema: str | None = ...) -> str:
    """
    Get the SQL db table schema for the given frame.

    Parameters
    ----------
    frame : DataFrame
    name : str
        name of SQL table
    keys : string or sequence, default: None
        columns to use a primary key
    con: an open SQL database connection object or a SQLAlchemy connectable
        Using SQLAlchemy makes it possible to use any DB supported by that
        library, default: None
        If a DBAPI2 object, only sqlite3 is supported.
    dtype : dict of column name to SQL type, default None
        Optional specifying the datatype for columns. The SQL type should
        be a SQLAlchemy type, or a string for sqlite3 fallback connection.
    schema: str, default: None
        Optional specifying the schema to be used in creating the table.

        .. versionadded:: 1.2.0
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numba
import numpy as np

"""
Numba 1D var kernels that can be shared by
* Dataframe / Series
* groupby
* rolling / expanding

Mirrors pandas/_libs/window/aggregation.pyx
"""
@numba.jit(nopython=True, nogil=True, parallel=False)
def add_var(val: float, nobs: int, mean_x: float, ssqdm_x: float, compensation: float, num_consecutive_same_value: int, prev_value: float) -> tuple[int, float, float, float, int, float]:
    ...

@numba.jit(nopython=True, nogil=True, parallel=False)
def remove_var(val: float, nobs: int, mean_x: float, ssqdm_x: float, compensation: float) -> tuple[int, float, float, float]:
    ...

@numba.jit(nopython=True, nogil=True, parallel=False)
def sliding_var(values: np.ndarray, start: np.ndarray, end: np.ndarray, min_periods: int, ddof: int = ...) -> np.ndarray:
    ...

